;Installer script for foxMod

RequestExecutionLevel user
SetCompressor /SOLID lzma
!include "MUI.nsh"
!define nameversion "foxMod v0.9"
!define version "0.9.0.0"
!define icon "foxMod.ico"

Name "${nameversion}"
OutFile "foxMod.exe"
InstallDir "$DOCUMENTS\My Games\Unreal Tournament 3\UTGame\"
Caption "${nameversion} Installer"
VIProductVersion "${version}"
VIAddVersionKey ProductName "${nameversion}"
VIAddVersionKey Comments "The installer for foxMod; see http://www.moddb.com/mods/foxmod-splitscreen for more information."
VIAddVersionKey FileDescription "foxMod Installer"
VIAddVersionKey FileVersion "${version}"
VIAddVersionKey ProductVersion "${version}"
VIAddVersionKey LegalCopyright "Creative Commons Attribution 4.0 International License - http://creativecommons.org/licenses/by/4.0/deed.en_US"
Icon "${icon}"
UninstallIcon "${icon}"
!define MUI_ICON "${icon}"

!define MUI_WELCOMEPAGE_TEXT "${nameversion} for Unreal Tournament 3 features: \n-- Improved AI\n-- Hor+ FOV widescreen support\n-- Proper splitscreen support\n-- Improved gamepad support\n-- ... and much more!\n\nfoxMod for UT3 is implemented as a collection of mutators (with optional menu edits for client-side features) and is fully compatible with vanilla UT3 clients over network games.\n\n** NOTE **\nIt is recommended you remove any old foxMod installations before continuing.\nYou will be automatically prompted to remove old installations if they were installed using the installer."
!define MUI_DIRECTORYPAGE_TEXT_TOP "The path to your UT3 UTGame folder should be displayed below. If it is incorrect, browse to the correct location. DO NOT select your UT3 install folder."
!define MUI_DIRECTORYPAGE_TEXT_DESTINATION "UTGame Folder"
!insertmacro MUI_PAGE_WELCOME

;;License page masquerading as a patch installation page
!define MUI_PAGE_HEADER_TEXT "Required Patches"
!define MUI_PAGE_HEADER_SUBTEXT "The Unreal Tournament III v2.1 patch is required."
!define MUI_LICENSEPAGE_TEXT_TOP "Please install the following patches before continuing."
!define MUI_LICENSEPAGE_TEXT_BOTTOM "If you have installed the v2.1 patch, click the $\"Good to go$\" button below. You must have the v2.1 patch installed to use foxMod."
!define MUI_LICENSEPAGE_BUTTON "Good to go!"
!insertmacro MUI_PAGE_LICENSE "Patches.rtf"

!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES

!define MUI_FINISHPAGE_NOAUTOCLOSE
!define MUI_FINISHPAGE_SHOWREADME
!define MUI_FINISHPAGE_SHOWREADME_TEXT "View Readme"
!define MUI_FINISHPAGE_SHOWREADME_FUNCTION "ShowReadme"
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_LANGUAGE "English"

UninstPage uninstConfirm
UninstPage instfiles


Section -AutoUninst
	IfFileExists $INSTDIR\.hg devProtect

	IfFileExists "$INSTDIR\foxMod Uninstall.exe" uninst

	;Delete really old files from pre-0.7 days
	Delete $INSTDIR\Published\CookedPC\Script\UTMutator_fox*.u

	;Uninstallation Registry values
	WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\foxModUT3" "DisplayName" "${nameversion} for Unreal Tournament 3"
	WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\foxModUT3" "UninstallString" '"$INSTDIR\foxMod Uninstall.exe"'
	WriteRegDWORD HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\foxModUT3" "NoModify" 1
	WriteRegDWORD HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\foxModUT3" "NoRepair" 1
	WriteUninstaller "foxMod Uninstall.exe"

	Return

uninst:
	ExecWait '"$INSTDIR\foxMod Uninstall.exe" _?=$INSTDIR' $0
	${If} $0 != 0
		MessageBox MB_OK "Installation aborted due to uninstaller error or abortion - please manually remove any old foxMod installations and try again."
		Quit
	${EndIf}

	Return

devProtect:
	MessageBox MB_OK "Installation aborted due to presence of .hg directory - please make sure this UTGame directory is not under development and try again."
	Quit
SectionEnd


Section "Mutators & Readme" secCore
SectionIn RO
	SetOutPath $INSTDIR
	File "foxMod Manual Install.txt"
	File "foxMod Readme.txt"

	SetOutPath $INSTDIR\Config
	File Config\UTMutator_fox*.ini

	SetOutPath $INSTDIR\Published\CookedPC\Script
	File Published\CookedPC\Script\foxMod.u
	File Published\CookedPC\Script\foxMod_AI.u

	SetOutPath $INSTDIR\Published\CookedPC\Sounds
	File Published\CookedPC\Sounds\A_Music_GoDown_Fix.upk
SectionEnd


Section "Client-Side Features" secClientStuff
	SetOutPath $INSTDIR\Localization\INT
	File Localization\INT\UTGameUI.int

	;File modifications
	SetOutPath $INSTDIR\Config
	Rename UTGame.ini UTGame.ini.foxBackup
	Rename UTInput.ini UTInput.ini.foxBackup
	Call EditUTEngine
	Delete $INSTDIR\Config\UTEngine.ini
	Rename UTEngine.ini.fox UTEngine.ini

	File Config\Default*.ini
SectionEnd


Section /o "CBP3 Dom. Support" secCBPSupport
	SetOutPath $INSTDIR\Published\CookedPC\Script
	File Published\CookedPC\Script\foxMod_CBP.u
	File Published\CookedPC\Script\foxMod_AI_CBP.u
SectionEnd


Section /o "Source Code" secSrcCode
	SetOutPath $INSTDIR\Src
	File /r Src\foxMod*
	File "Src\Compile All.bat"
	File "Src\Compile All.vbs"
	File Src\Todo.txt
	SetOutPath $INSTDIR\Src\OtherFiles\Unpublished\CookedPC\Sounds
	File Unpublished\CookedPC\Sounds\A_Music_GoDown_Fix.upk
	SetOutPath $INSTDIR\Src\OtherFiles
	File *.hg*
	File *.bat
	File *.ico
	File *.nsi
	File *.rtf
SectionEnd


Section /o "Start Menu Prog. Group" secProgGroup
	CreateDirectory $SMPROGRAMS\foxMod-UT3
	CreateShortCut "$SMPROGRAMS\foxMod-UT3\Uninstall.lnk" "$INSTDIR\foxMod Uninstall.exe" "" "$INSTDIR\foxMod Uninstall.exe" 0
	CreateShortCut "$SMPROGRAMS\foxMod-UT3\ReadMe.lnk" "$INSTDIR\foxMod Readme.txt" "" "$INSTDIR\foxMod Readme.txt" 0
SectionEnd


!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
	!insertmacro MUI_DESCRIPTION_TEXT ${secCore} "foxMod mutators and documentation."
	!insertmacro MUI_DESCRIPTION_TEXT ${secClientStuff} "foxMod client-side features including FOV, splitscreen and improved gamepad support. Required ONLY if you would like to use these features; uncheck this to install foxMod mutators only (good for dedicated servers)."
	!insertmacro MUI_DESCRIPTION_TEXT ${secCBPSupport} "foxMod extensions to support the new Domination gametype included with Community Bonus Pack 3 (CBP3). Required ONLY if you use CBP3 Domination."
	!insertmacro MUI_DESCRIPTION_TEXT ${secSrcCode} "foxMod source code pre-extracted into the UTGame\Src directory."
	!insertmacro MUI_DESCRIPTION_TEXT ${secProgGroup} "Start Menu folder containing shortcuts to foxMod's readme and uninstaller."
!insertmacro MUI_FUNCTION_DESCRIPTION_END


Section "un.Uninstall"
	IfFileExists $INSTDIR\.hg devProtect

	DeleteRegKey HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\foxModUT3"

	IfFileExists $INSTDIR\Config\UTGame.ini.foxBackup check
	Goto cont

check:
	IfFileExists $INSTDIR\Config\UTInput.ini.foxBackup uninstClFt

cont:
	Delete $INSTDIR\Config\UTMutator_fox*.ini
	Delete $INSTDIR\Localization\INT\UTGameUI.int
	Delete $INSTDIR\Published\CookedPC\Script\foxMod*.u
	Delete $INSTDIR\Published\CookedPC\Sounds\A_Music_GoDown_Fix.upk
	Delete "$INSTDIR\foxMod Manual Install.txt"
	Delete "$INSTDIR\foxMod Readme.txt"

	RMDir /r $SMPROGRAMS\foxMod-UT3

	RMDir /r $INSTDIR\Src\foxMod
	RMDir /r $INSTDIR\Src\foxMod_AI
	RMDir /r $INSTDIR\Src\foxMod_AI_CBP
	RMDir /r $INSTDIR\Src\foxMod_CBP
	Delete "$INSTDIR\Src\Compile All.bat"
	Delete "$INSTDIR\Src\Compile All.vbs"
	Delete "$INSTDIR\Src\foxMod Installer.nsi"
	Delete "$INSTDIR\Src\Package All.bat"
	Delete $INSTDIR\Src\Patches.rtf
	Delete $INSTDIR\Src\Todo.txt

	Delete "$INSTDIR\foxMod Uninstall.exe"

	Return

uninstClFt:
	Delete $INSTDIR\Config\Default*.ini
	Delete $INSTDIR\Config\UTGame.ini
	Delete $INSTDIR\Config\UTInput.ini

	;File modifications
	SetOutPath $INSTDIR\Config
	Rename UTGame.ini.foxBackup UTGame.ini
	Rename UTInput.ini.foxBackup UTInput.ini
	Call un.RestoreUTEngine
	Delete $INSTDIR\Config\UTEngine.ini
	Rename UTEngine.ini.fox UTEngine.ini

	Goto cont

devProtect:
	MessageBox MB_OK "Uninstallation aborted due to presence of .hg directory - please make sure this UTGame directory is not under development and try again."
	Quit
SectionEnd


;==== Utility functions ====

Function ShowReadme
	ExecShell "open" "$INSTDIR\foxMod Readme.txt"
FunctionEnd


Function EditUTEngine
start:
	ClearErrors
	;Open files
	FileOpen $0 UTEngine.ini r
	FileOpen $1 UTEngine.ini.fox w
	IfErrors onError

lineLoop:
	;Read lines and check if they match ones we want to edit
	FileRead $0 $9
	IfErrors close
	StrCmpS $9 "ProfileSettingsClassName=UTGame.UTProfileSettings$\r$\n" ProfileSettings
	StrCmpS $9 "GameViewportClientClassName=UTGame.UTGameViewportClient$\r$\n" GameViewportClient
	StrCmpS $9 "GlobalDataStoreClasses=UTGame.UTDataStore_GameSettingsDM$\r$\n" GameSettings
	FileWrite $1 $9
	Goto lineLoop
ProfileSettings:
	StrCpy $9 ";$9"
	FileWrite $1 $9
	FileWrite $1 "ProfileSettingsClassName=foxMod.foxProfileSettings$\r$\n"
	Goto lineLoop
GameViewportClient:
	StrCpy $9 ";$9"
	FileWrite $1 $9
	FileWrite $1 "GameViewportClientClassName=foxMod.foxGameViewportClient$\r$\n"
	Goto lineLoop
GameSettings:
	StrCpy $9 ";$9"
	FileWrite $1 $9
	FileWrite $1 "GlobalDataStoreClasses=foxMod.foxDataStore_GameSettingsDM$\r$\n"
	Goto lineLoop

close:
	;Close files
	FileClose $1
	FileClose $0
	Return

onError:
	MessageBox MB_RETRYCANCEL "UTEngine.ini could not be opened. If you cancel, please perform the edits manually as detailed in foxMod Manual Install.txt" IDRETRY start
FunctionEnd


Function un.RestoreUTEngine
start:
	ClearErrors
	;Open files
	FileOpen $0 UTEngine.ini r
	FileOpen $1 UTEngine.ini.fox w
	IfErrors onError

lineLoop:
	;Read lines and check if they match ones we want to edit
	FileRead $0 $9
	IfErrors close
	StrCmpS $9 ";ProfileSettingsClassName=UTGame.UTProfileSettings$\r$\n" Uncomment
	StrCmpS $9 ";GameViewportClientClassName=UTGame.UTGameViewportClient$\r$\n" Uncomment
	StrCmpS $9 ";GlobalDataStoreClasses=UTGame.UTDataStore_GameSettingsDM$\r$\n" Uncomment
	StrCmpS $9 "ProfileSettingsClassName=foxMod.foxProfileSettings$\r$\n" lineLoop
	StrCmpS $9 "GameViewportClientClassName=foxMod.foxGameViewportClient$\r$\n" lineLoop
	StrCmpS $9 "GlobalDataStoreClasses=foxMod.foxDataStore_GameSettingsDM$\r$\n" lineLoop
	FileWrite $1 $9
	Goto lineLoop
Uncomment:
	StrCpy $9 $9 "" 1
	FileWrite $1 $9
	Goto lineLoop

close:
	;Close files
	FileClose $1
	FileClose $0
	Return

onError:
	MessageBox MB_RETRYCANCEL "UTEngine.ini could not be opened. If you cancel, please perform the edits manually as detailed in foxMod Manual Install.txt" IDRETRY start
FunctionEnd
