/**
 * A mutator that replaces the UTGame.UTPlayerController class with a splitscreen profile friendly PC class ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class UTMutator_foxSplitScreen extends UTMutator
	config(Mutator_foxSplitScreen);

var array<string> ActivationErrors;
var class<Object> OldPlayerControllerClass;
var class<Object> OldDefaultPawnClass;
var class<Object> OldDeathMessageClass;

var globalconfig bool bForceActivate;
var globalconfig bool bSkipActivationErrorBroadcast;
var globalconfig bool bClampAspectRatioFOV;
var globalconfig float ServerMinFOV;
var globalconfig float ServerMaxFOV;

function InitMutator(string Options, out string ErrorMessage)
{
	local UTGame UTG;

	Super.InitMutator(Options, ErrorMessage);

	UTG = UTGame(WorldInfo.Game);
	if (UTG != None) {
		//Handle controllers
		OldPlayerControllerClass = UTG.PlayerControllerClass;
		if (UTG.PlayerControllerClass == class'UTMissionPlayerController') {
			UTG.PlayerControllerClass = class'foxMissionPlayerController';
			UTG.ConsolePlayerControllerClass = class'foxMissionPlayerController';
		}
		else if (bForceActivate || UTG.PlayerControllerClass == class'UTPlayerController') {
			UTG.PlayerControllerClass = class'foxPlayerController';
			UTG.ConsolePlayerControllerClass = class'foxPlayerController';
		}

		//Handle pawns
		OldDefaultPawnClass = UTG.DefaultPawnClass;
		if (UTG.DefaultPawnClass == class'UTHeroPawn')
			UTG.DefaultPawnClass = class'foxHeroPawn';
		else if (UTG.DefaultPawnClass == class'UTBetrayalPawn')
			UTG.DefaultPawnClass = class'foxBetrayalPawn';
		else if (bForceActivate || UTG.DefaultPawnClass == class'UTPawn')
			UTG.DefaultPawnClass = class'foxPawn';

		//Handle death messages
		OldDeathMessageClass = UTG.DeathMessageClass;
		if (bForceActivate || UTG.DeathMessageClass == class'UTDeathMessage' || UTG.DeathMessageClass == class'UTTeamDeathMessage')
			UTG.DeathMessageClass = class'foxDeathMessage';

		//Handle voting (if not previously configured by user)
		if (UTG.VoteCollectorClassName == "")
			UTG.VoteCollectorClassName = "foxMod.foxVoteCollector";
	}

	SetTimer(4.0, false, 'CheckErrors');
}

//fox: Check any activation errors
function CheckErrors()
{
	local UTGame UTG;

	UTG = UTGame(WorldInfo.Game);
	if (UTG == None) {
		ThrowError("GameInfo missing! Something is", WorldInfo.Game, "Everything");
		return;
	}

	if (UTG.PlayerControllerClass == OldPlayerControllerClass)
		ThrowError("PlayerController", UTG.PlayerControllerClass, "FOV, gamepad and splitscreen settings");
	if (UTG.DefaultPawnClass == OldDefaultPawnClass)
		ThrowError("Pawn", UTG.DefaultPawnClass, "Network sound fixes and weapon tossing");
	if (UTG.DeathMessageClass == OldDeathMessageClass)
		ThrowError("DeathMessage", UTG.DeathMessageClass, "Team-colored death messages");
	if (UTG.VoteCollectorClassName != "foxMod.foxVoteCollector")
		ThrowError("VoteCollector", UTG.VoteCollectorClassName, "Splitscreen voting");

	SetTimer(1.0, false, 'BroadcastErrors');
}
function ThrowError(string ClassName, coerce string Reason, string BrokenFunctions)
{
	local string Error;

	Error = "foxMod:" @ ClassName @ "incompatible with" @ Reason $ ".";
	ActivationErrors.AddItem(Error);
	`Log(Error);

	Error = "        --" @ BrokenFunctions @ "will not function.";
	ActivationErrors.AddItem(Error);
	`Log(Error);
}
function BroadcastErrors()
{
	local PlayerController C;
	local string E;

	if (bSkipActivationErrorBroadcast || ActivationErrors.Length < 1)
		return;

	foreach WorldInfo.AllControllers(class'PlayerController', C) {
		foreach ActivationErrors(E)
			C.ClientMessage(E);
		C.ClientMessage("foxMod: One or more errors have occurred. Please check the console (~ key) for more info.");
	}
}

//fox: Clamp FOV values for player
function NotifyLogin(Controller NewPlayer)
{
	//HACK Using Timer to avoid extension typecasting (CBP etc.)
	if (UTPlayerController(NewPlayer) != None)
		UTPlayerController(NewPlayer).SetTimer(0.01, false, 'SetFOVClamps');

	Super.NotifyLogin(NewPlayer);
}

//fox: Check various replacements as needed
function bool CheckReplacement(Actor Other)
{
	local UTVehicle V;

	V = UTVehicle(Other);
	if (V != None) {
		//Handle extended look-to-steer settings
		class'foxPlayerController'.static.SetLookToSteerSettings(V);

		//Also fix Hellbender VWeap
		if (UTVehicle_HellBender_Content(V) != None
		&& V.Seats[1].GunClass == class'UTVWeap_HellBenderPrimary')
			V.Seats[1].GunClass = class'foxVWeap_HellBenderPrimary';
		//.. and the preposterous Fury custom entry radius
		else if (UTVehicle_Fury(V) != None)
			V.bHasCustomEntryRadius = false;
	}

	return Super.CheckReplacement(Other);
}

defaultproperties
{
	GroupNames[0]="FOXMOD"
	bForceActivate=False
	bSkipActivationErrorBroadcast=False
	bClampAspectRatioFOV=False
	ServerMinFOV=60.0
	ServerMaxFOV=130.0
}
