/**
 * New announcer for managing splitscreen announcements ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxAnnouncer extends UTAnnouncer;

//fox: Currently playing PRI (checked by foxPlayerController)
var PlayerReplicationInfo PlayingAnnouncementPRI;

//fox: Voice pitch variance
var config float VoicePitch;
var config float VoicePitchVariance;

//fox: Sanity-check our config variables and such
function PostBeginPlay()
{
	Super.PostBeginPlay();

	VoicePitch = FMax(0.000001, VoicePitch);
	VoicePitchVariance = FMax(0, VoicePitchVariance);

	SaveConfig();
}

//fox: Play announcement, optionally displaying portrait for self / local players (if AnnounceData is present)
function PlayAnnouncementNow(class<UTLocalMessage> InMessageClass, int MessageIndex, optional PlayerReplicationInfo PRI, optional Object OptionalObject)
{
	local SoundNodeWave ASound;
	local bool bUsingVoiceCue;
	local UTHUD HUD;

	//We've probably "extended" OptionalObject, which will now be held in OptionalObject.OriginalObject, so pass that in if appropriate
	ASound = InMessageClass.Static.AnnouncementSound(MessageIndex,
		(foxAnnounceData(OptionalObject) != None) ? foxAnnounceData(OptionalObject).OriginalObject : OptionalObject,
		PlayerOwner);

	if (ASound != None) {
		if (CurrentAnnouncementComponent != none) {
			if (PlayerOwner != none)
				PlayerOwner.DetachComponent(CurrentAnnouncementComponent);
			CurrentAnnouncementComponent = none;
		}

		//@FIXME: part of playsound pool?
		// if we are a UTVoice when we want to use the special UTVoiceSoundCue which is in the correct SoundGroup (i.e. effects)
		HUD = UTHUD(PlayerOwner.myHUD);
		if (HUD != None && HUD.bIsSplitScreen && !HUD.bIsFirstPlayer && ClassIsChildOf(InMessageClass, class'UTScriptedVoiceMessage'))
			CurrentAnnouncementComponent = None;
		else if (ClassIsChildOf(InMessageClass, class'UTVoice') || ClassIsChildOf(InMessageClass, class'UTScriptedVoiceMessage')) {
			CurrentAnnouncementComponent = PlayerOwner.CreateAudioComponent(UTVoiceSoundCue, false, false);
			bUsingVoiceCue = TRUE;
		}
		else {
			CurrentAnnouncementComponent = PlayerOwner.CreateAudioComponent(AnnouncerSoundCue, false, false);
			bUsingVoiceCue = FALSE;
		}

		// CurrentAnnouncementComponent will be none if -nosound option used
		if (CurrentAnnouncementComponent != None) {
			CurrentAnnouncementComponent.SetWaveParameter('Announcement', ASound);
			if(bUsingVoiceCue) {
				UTVoiceSoundCue.Duration = ASound.Duration;
				UTVoiceSoundCue.VolumeMultiplier = InMessageClass.Default.AnnouncementVolume;
				UTVoiceSoundCue.PitchMultiplier = VoicePitch + FRand() * VoicePitchVariance - VoicePitchVariance / 2;
			}
			else {
				AnnouncerSoundCue.Duration = ASound.Duration;
				AnnouncerSoundCue.VolumeMultiplier = InMessageClass.Default.AnnouncementVolume;
				AnnouncerSoundCue.PitchMultiplier = VoicePitch + FRand() * VoicePitchVariance - VoicePitchVariance / 2;
			}
			CurrentAnnouncementComponent.bAutoDestroy = true;
			CurrentAnnouncementComponent.bShouldRemainActiveIfDropped = true;
			CurrentAnnouncementComponent.bAllowSpatialization = false;
			CurrentAnnouncementComponent.bAlwaysPlay = TRUE;
			CurrentAnnouncementComponent.Play();
		}
		PlayingAnnouncementClass = InMessageClass;
		PlayingAnnouncementIndex = MessageIndex;
		PlayingAnnouncementPRI = PRI;

		// NOTE: Audio always plays back in real-time, so we'll scale our duration by the world's time dilation
		SetTimer(ASound.Duration * WorldInfo.TimeDilation + 0.05, false, 'AnnouncementFinished');

		//TODO get better method for bShowPortraits and bShowOwnPortrait as those will eventually be a non-config player preference
		if (HUD != None && InMessageClass.default.bShowPortrait && UTPlayerReplicationInfo(PRI) != None) {
			if (foxAnnounceData(OptionalObject) != None)
				foreach foxAnnounceData(OptionalObject).HUDs(HUD)
					HUD.ShowPortrait(UTPlayerReplicationInfo(PRI), ASound.Duration, true);
			else if (class'foxPlayerController'.default.bShowPortraits
			&& (class'foxPlayerController'.default.bShowOwnPortrait || PRI != PlayerOwner.PlayerReplicationInfo))
				HUD.ShowPortrait(UTPlayerReplicationInfo(PRI), ASound.Duration, true);
		}
	}
	else {
		//`log("NO SOUND FOR "$InMessageClass@MessageIndex@OptionalObject@OptionalObject.name);
		PlayNextAnnouncement();
	}
}

//fox: Also support PlayingAnnouncementPRI
function PlayNextAnnouncement()
{
	local UTQueuedAnnouncement PlayedAnnouncement;

	PlayingAnnouncementClass = None;
	if (Queue != None) {
		PlayedAnnouncement = Queue;
		Queue = PlayedAnnouncement.nextAnnouncement;
		PlayAnnouncementNow(PlayedAnnouncement.AnnouncementClass, PlayedAnnouncement.MessageIndex, PlayedAnnouncement.PRI, PlayedAnnouncement.OptionalObject);
		PlayingAnnouncementClass = PlayedAnnouncement.AnnouncementClass;
		PlayingAnnouncementIndex = PlayedAnnouncement.MessageIndex;
		PlayingAnnouncementPRI = PlayedAnnouncement.PRI;
		PlayedAnnouncement.Destroy();
	}
}

defaultproperties
{
	VoicePitch = 1.0;
	VoicePitchVariance = 0.05;
}
