/**
 * A ViewportClient that handles new splitscreen functionality ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxGameViewportClient extends UTGameViewportClient
	config(Game);

var int numLocalPlayers;

var float resX;
var float resY;

var string CMTravelMethod;
var bool bNeedsMutatorInteraction;

//fox: The best thing Bethesda's done since Valve's "qiut"
exec function qqq()
{
	ConsoleCommand("Quit");
}

//fox: Retrieve an accurate number of local splitscreen players
event UpdateNumLocalPlayers()
{
	local int i;

	numLocalPlayers = 1;
	for (i = 1; i < 4; i++)
		if (FindPlayerByControllerId(i) != None)
			numLocalPlayers++;
}

//fox: Hijack CreatePlayer and RemovePlayer to reliably update numLocalPlayers
event LocalPlayer CreatePlayer(int ControllerId, out string OutError, bool bSpawnActor)
{
	local LocalPlayer LP;

	LP = Super.CreatePlayer(ControllerId, OutError, bSpawnActor);
	UpdateNumLocalPlayers();

	return LP;
}
event bool RemovePlayer(LocalPlayer ExPlayer)
{
	local bool bSuccess;

	bSuccess = Super.RemovePlayer(ExPlayer);
	UpdateNumLocalPlayers();

	return bSuccess;
}

//fox: Hook rendering to also report our current resolution
event PostRender(Canvas Canvas)
{
	Super.PostRender(Canvas);

	//Set our last known resolution to resX x resY
	resX = Canvas.SizeX;
	resY = Canvas.SizeY;

	//HACK Launch Mission Select with mutator support if needed
	if (bNeedsMutatorInteraction)
		LaunchMissionSelectMutators();
}

//fox: When game ends, reset the campaign mutator flag
event GameSessionEnded()
{
	local WorldInfo WI;

	WI = GetCurrentWorldInfo();
	if (WI != None && WI.NetMode != NM_Client && Left(WI.GetMapName(true), 10) == "UTFrontEnd")
		CMTravelMethod = default.CMTravelMethod;
	bNeedsMutatorInteraction = true;

	Super.GameSessionEnded();
}

//fox: If we're in the mission selection screen, re-launch it with mutators enabled
exec function LaunchMissionSelectMutators()
{
	local WorldInfo WI;

	WI = GetCurrentWorldInfo();
	if (WI != None && WI.NetMode != NM_Client && UTMissionSelectionGame(WI.Game) != None && foxMissionSelectionGame(WI.Game) == None) {
		//TODO HACK: ServerTravel does not work for splitscreen, messes up very strangely
		if (numLocalPlayers > 1)
			CMTravelMethod = default.CMTravelMethod;
		`Log("foxLog: GVC Mutate Campaign Hook");
		ConsoleCommand(CMTravelMethod @ WI.GetMapName(true) $ "?Listen?Game=foxMod.foxMissionSelectionGame?Mutator=" $ class'foxMissionSelectionGame'.default.CampaignMutators);
		CMTravelMethod = "ServerTravel";
	}

	bNeedsMutatorInteraction = false;
}

//fox: Override for 0 = Horizontal, > 1 = Vertical
exec function SetSplit(int mode)
{
	switch (mode) {
		case 0:
			SetSplitscreenConfiguration(eSST_NONE);
			break;
		case 1:
		case 2:
			SetSplitscreenConfiguration(eSST_2P_VERTICAL);
			break;
		default: //3: 4:
			SetSplitscreenConfiguration(eSST_3P_FAVOR_BOTTOM);
	}
}

defaultproperties
{
	numLocalPlayers=1 //Actually 0 before first player is created, but just to be safe as numLocalPlayers is accurately updated
	resX=1024
	resY=768
	CMTravelMethod="Open"
	bNeedsMutatorInteraction=True
	//Split overrides for dual monitor support
	SplitscreenInfo={(
		(PlayerData=((SizeX=1.0f,SizeY=1.0f,OriginX=0.0f,OriginY=0.0f))),	// eSST_NONE
		(PlayerData=((SizeX=1.0f,SizeY=0.5f,OriginX=0.0f,OriginY=0.0f),		// eSST_2P_HORIZONTAL - Default
					(SizeX=1.0f,SizeY=0.5f,OriginX=0.0f,OriginY=0.5f))),
		(PlayerData=((SizeX=0.5f,SizeY=1.0f,OriginX=0.0f,OriginY=0.0f),		// eSST_2P_VERTICAL - MultiMonitor
					(SizeX=0.5f,SizeY=1.0f,OriginX=0.5f,OriginY=0.0f))),
		(PlayerData=((SizeX=1.0f,SizeY=0.5f,OriginX=0.0f,OriginY=0.0f),		// eSST_3P_FAVOR_TOP - Default
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.0f,OriginY=0.5f),
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.5f,OriginY=0.5f))),
		(PlayerData=((SizeX=0.5f,SizeY=0.5f,OriginX=0.0f,OriginY=0.0f),		// eSST_3P_FAVOR_BOTTOM - MultiMonitor (Vertical)
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.0f,OriginY=0.5f),
					(SizeX=0.5f,SizeY=1.0f,OriginX=0.5f,OriginY=0.0f))),
		(PlayerData=((SizeX=0.5f,SizeY=0.5f,OriginX=0.0f,OriginY=0.0f),		// eSST_4P
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.5f,OriginY=0.0f),
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.0f,OriginY=0.5f),
					(SizeX=0.5f,SizeY=0.5f,OriginX=0.5f,OriginY=0.5f)))
	)}
}
