/**
 * Hacked up MissionSelectionGame to allow mutators in campaign missions ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */

class foxMissionSelectionGame extends UTMissionSelectionGame
	config(Game);

var config string CampaignMutators;

//fox: Copypasta from UTMissionSelectionGame with a key addition of appending a mutator string to the URL
function AcceptMission()
{
	local UTMissionGRI MGRI;
	local string URL, GameType;
	local EMissionInformation Mission;
	local int i;
	local UTPlayerController PC;
	local UTProfileSettings Profile;
	local UTBot B;

	// Log any bots still hanging around
	foreach WorldInfo.AllControllers(class'UTBot', B) {
		`Warn("BOT STILL AROUND!" @ B.PlayerReplicationInfo.PlayerName @ B);
		//@hack: pretty sure the right fix is to prevent Logout() from adding a bot to replace an exiting human
		//	when we haven't accepted the mission yet, but this is a safer fix at this point
		B.Destroy();
	}

	MGRI = UTMissionGRI(GameReplicationInfo);
	if (MGRI != None && MGRI.GetCurrentMission(Mission)) {

		UTGameUISceneClient(class'UIRoot'.static.GetSceneClient()).MissionText = " DEBUG Prev. MissionID: "$SinglePlayerMissionID$" \n DEBUG MissionID: "$Mission.MissionID$" \n\ "$Mission.BriefingText;

		GameType = ParseOption(Mission.URL, "game");
		// default to Team DM if no gametype specified for DM maps
		if (GameType == "" && Left(Mission.Map, 3) ~= "DM-") {
			Mission.URL $= "?game=UTGame.UTTeamGame";
			bNextMissionTeamGame = true;
		}
		else {
			//@hack: we can't load the game class on console, so assume deathmatch is the only FFA type
			bNextMissionTeamGame = !(GameType ~= "utgame.utdeathmatch");
		}

		Profile = UTProfileSettings(HostPlayer.OnlinePlayerData.ProfileProvider.Profile);

		// Always read the skill level from the profile....
		GameDifficulty = Profile.GetCampaignSkillLevel() * 2;

		URL = Mission.Map $ Mission.URL $ "?SPI="$Mission.MissionID$"?PlayersMustBeReady=1?Difficulty="$GameDifficulty$"?MaxPlayers="$MaxPlayers;
		if (InStr(Caps(URL), "?TIMELIMIT=") == INDEX_NONE)
			URL $= "?TimeLimit=20";

		if (Profile == None || !Profile.HasPersistentKey(ESPKey_CanStealNecris))
			URL $= "?NecrisLocked=1";

		if (HostPlayer != None) {
			if (Mission.MissionID == 149) // We won, clear out the mission data in the profile.
				Profile.SetCurrentMissionData(INDEX_None, INDEX_None);

			if (MGRI.GameModifierCard != '')
				ProcessModifierCard(MGRI.GameModifierCard, Profile, URL, Mission);

			// Clear Any Must use cards
			for (i = 0; i < class'UTGameModifierCard'.default.Deck.Length; i++)
				if (class'UTGameModifierCard'.default.Deck[i].bMustUse)
					HostPlayer.UseModifierCard(class'UTGameModifierCard'.default.Deck[i].Tag);
		}

		// we need to call this twice so we first add all of the characters that must have customized meshes and then with everyone else
		AddBotsFromMissionData(Mission, true);
		AddBotsFromMissionData(Mission, false);

		// make sure all the humans are on the correct team
		foreach WorldInfo.AllControllers(class'UTPlayerController', PC)
			SetTeam(PC, bNextMissionTeamGame ? Teams[0] : None, false);

		//fox: lulz I'm hijacking your campaign
		URL $= "?Mutator="$CampaignMutators;
		`Log("foxLog: MSG AcceptMission"@URL);
		//Resume normal programming

		NextMissionURL = URL;
		MGRI.ChangeMenuState(EMS_Launch);
		GotoState('SyncClients');
	}
	else
		`log("Warning: The Single Play game could not accept a mission.");
}

//fox: InitGame that handles mutators, since UTEntryGame has a blank init block; copypasta from Game
event InitGame(string Options, out string ErrorMessage)
{
	local string InOpt, LeftOpt;
	local int pos;

	InOpt = ParseOption(Options, "Mutator");
	if (InOpt != "") {
		`log("Mutators"@InOpt);
		while (InOpt != "") {
			pos = InStr(InOpt,",");
			if (pos > 0) {
				LeftOpt = Left(InOpt, pos);
				InOpt = Right(InOpt, Len(InOpt) - pos - 1);
			}
			else {
				LeftOpt = InOpt;
				InOpt = "";
			}
			AddMutator(LeftOpt, true);
		}
	}

	if (BaseMutator != none)
		BaseMutator.InitMutator(Options, ErrorMessage);
}

defaultproperties
{
	CampaignMutators="foxMod_AI.foxMod_AI,foxMod.foxMod"
}
