/**
 * New music manager for managing phat beats ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxMusicManager extends UTMusicManager;

//fox: CurrTempo / CurrFadeFactor is private, so we must re-implement them
var float Tempo;
var float FadeFactor;

//fox: Configurable music switch times
var config float MinActionTime; //-1 = pretend action is always going on
var config float MaxActionTime; //-1 = skip action music
var config float MaxTensionTime; //-1 = skip tension music when winding down to ambient

//fox: Force tension/suspense music when requested (similar to bPendingAction)
var bool bPendingTension;

//fox: For netclient PoweredUp support
var UTTimedPowerup LastTimedPowerup;

//fox: Pawn.PoweredUp() is not reliable as FireRateMultiplier etc. aren't replicated, so just manually check inventory
function bool PawnPoweredUp(Pawn P)
{
	if (LastTimedPowerup != None
	&& LastTimedPowerup.Owner == P)
		return true;

	LastTimedPowerup = None;
	if (P.InvManager != None)
		foreach P.InvManager.InventoryActors(class'UTTimedPowerup', LastTimedPowerup)
			return true;

	//No powerup? Check vehicle driver too
	if (Vehicle(P) != None)
		P = Vehicle(P).Driver;
	if (P != None && P.InvManager != None)
		foreach P.InvManager.InventoryActors(class'UTTimedPowerup', LastTimedPowerup)
			return true;

	return false;
}

//fox: Set the new Tempo/FadeFactor variables since the original ones were private in UTMusicManager
function StartMusic()
{
	local UTMapInfo UMI;

	PlayerOwner = UTPlayerController(Owner);
	CurrentState = MST_Ambient;

	UMI = UTMapInfo(WorldInfo.GetMapInfo());

	if (UMI != None && UMI.MapMusicInfo != None) {
		//We're gonna also take this moment to fix GoDown's non-looping action track - aww yeah
		if (UMI.MapMusicInfo == UTMapMusicInfo'A_Music_Arrangements.Tracks.A_Music_GoDown_DM')
			UMI.MapMusicInfo = UTMapMusicInfo'A_Music_GoDown_Fix.Tracks.A_Music_GoDown_DM_Fix';

		//If our intro track matches some other defined music, just change states so it will be able to stop
		switch UMI.MapMusicInfo.MapMusic.Intro.TheCue {
			case UMI.MapMusicInfo.MapMusic.Victory.TheCue:
				SetTrack(MST_Victory); break;
			case UMI.MapMusicInfo.MapMusic.Action.TheCue:
				SetTrack(MST_Action); break;
			case UMI.MapMusicInfo.MapMusic.Suspense.TheCue:
				SetTrack(MST_Suspense); break;
			case UMI.MapMusicInfo.MapMusic.Tension.TheCue:
				SetTrack(MST_Tension); break;
			case UMI.MapMusicInfo.MapMusic.Ambient.TheCue:
				SetTrack(MST_Ambient); break;
			default:
				SetTrack(MST_Max);
				CurrentState = MST_Ambient; //Because Intro is techincally Ambient state and MST_Max is of course higher than MST_Action
		}
	}

	LastActionEventTime = WorldInfo.TimeSeconds;

	//Sanity-check our config variables
	if (MinActionTime != -1)
		MinActionTime = FMax(0, MinActionTime);
	if (MaxActionTime != -1)
		MaxActionTime = FMax(MinActionTime, MaxActionTime);
	if (MaxTensionTime != -1)
		MaxTensionTime = FMax(MinActionTime, MaxTensionTime);

	SaveConfig();
}

//fox: Process PendingEvent correctly, allowing EventIndex 0 to actually fire and preventing overlap
function MusicEvent(int NewEventIndex)
{
	/* NewEventIndex: (+/- = force action/tension music respectively, otherwise just keep it going)
		-2	- firing weapon
		-1+	- being shot at (but not hit) or taking damage from enemy
		0-	- being watched by enemy -- TODO currently unused as it's a bit sensitive
		1	- kill
		2	- died
		3	- returned flag
		4	- enemy took flag (from base)
		5	- enemy returned flag NOT IMPLEMENTED, CHANGE?
		6+	- major kill (first blood, killed flag carrier, or took lead in DM)
		7	- took flag
		8+	- killing spree
		9+	- double kill
		10+	- long spree
		11+	- ultrakill
		12+	- monster kill
		13	- score behind
		14	- score increase lead
		15	- score take lead
	*/

	// set pendingevent - will be processed on the next beat
	if (NewEventIndex > PendingEvent) {
		PendingEvent = NewEventIndex;
		PendingEventPlayTime = WorldInfo.TimeSeconds + PendingEventDelay;
	}

	// request change to action track if appropriate (treated like tension if action music disabled)
	//if (CurrentState < ((MaxActionTime > 0) ? MST_Action : MST_Tension)) { <--- silly compiler can't compile that
	if ((MaxActionTime > 0) ? CurrentState < MST_Action : CurrentState < MST_Tension) {
		switch (PendingEvent) {
			case -1:
			case 6:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12:
				if (MaxActionTime > 0)
					bPendingAction = true;
				else if (MaxTensionTime > 0)
					bPendingTension = true;
		}
	}

	// request change to tension track if appropriate
	if (CurrentState < MST_Tension) {
		switch (PendingEvent) {
			case 0:
				if (MaxTensionTime > 0)
					bPendingTension = true;
		}
	}

	LastActionEventTime = WorldInfo.TimeSeconds;
}

//fox: Support for bPendingTension and other things
function ProcessMusicEvent()
{
	local UTMapInfo UMI;
	local SoundCue EventCue;
	local AudioComponent EventTrack;

	// change to action track if appropriate
	if (bPendingAction) {
		SetTrack(MST_Action);
		bPendingAction = false;
	}

	// change to tension track if appropriate
	if (bPendingTension) {
		SetTrack(MST_Tension);
		bPendingTension = false;
	}

	UMI = UTMapInfo(WorldInfo.GetMapInfo());
	// means the content folks have not set the map up with music yet
	if (UMI == None) {
		//`log(GetFuncName() @ "UMI is None");
		return;
	}

	// play appropriate stinger
	switch (PendingEvent) {
		//Disable the kill stinger for now because it's annoying -- TODO hook up a player preference for this
		//case 1: EventCue = UMI.MapMusicInfo.MapStingers.Kill; break;
		case 2: EventCue = UMI.MapMusicInfo.MapStingers.Died; break;
		case 3: EventCue = UMI.MapMusicInfo.MapStingers.ReturnFlag; break;
		case 4: EventCue = UMI.MapMusicInfo.MapStingers.EnemyGrabFlag; break;
		case 5: EventCue = UMI.MapMusicInfo.MapStingers.FlagReturned; break;
		case 6: EventCue = UMI.MapMusicInfo.MapStingers.MajorKill; break;
		case 7: EventCue = UMI.MapMusicInfo.MapStingers.GrabFlag; break;
		case 8: EventCue = UMI.MapMusicInfo.MapStingers.FirstKillingSpree; break;
		case 9: EventCue = UMI.MapMusicInfo.MapStingers.DoubleKill; break;
		case 10: EventCue = UMI.MapMusicInfo.MapStingers.LongKillingSpree; break;
		case 11: EventCue = UMI.MapMusicInfo.MapStingers.MultiKill; break;
		case 12: EventCue = UMI.MapMusicInfo.MapStingers.MonsterKill; break;
		case 13: EventCue = UMI.MapMusicInfo.MapStingers.ScoreLosing; break;
		case 14: EventCue = UMI.MapMusicInfo.MapStingers.ScoreTie; break;
		case 15: EventCue = UMI.MapMusicInfo.MapStingers.ScoreWinning; break;
	}

	EventTrack = CreateNewTrack(EventCue);
	if (EventTrack != None) {
		EventTrack.bAutoDestroy = true;
		EventTrack.SoundCue.SoundGroup = 'Music';
		EventTrack.VolumeMultiplier = StingerVolumeMultiplier;
		EventTrack.Play();
	}
}

//fox: Handle local splitscreen players and some new stuff
function Tick(float DeltaTime)
{
	local float NumBeats;
	local EMusicState NewState, ConfirmedNewState;
	local UTPlayerController PC;
	local TeamInfo TI;
	local PlayerReplicationInfo PRI;
	local bool bOurCorePowered, bTheirCorePowered;
	local int i;

	// Cross-fade, ramp up current track
	if (CurrentTrack != None && CurrentTrack.VolumeMultiplier < MusicVolume)
		CurrentTrack.VolumeMultiplier = FMin(MusicVolume, CurrentTrack.VolumeMultiplier + FadeFactor*DeltaTime);

	for (i = 0; i < 6; i++) {
		// ramp down other tracks
		if (MusicTracks[i] != None && MusicTracks[i] != CurrentTrack && MusicTracks[i].VolumeMultiplier > 0.f) {
			MusicTracks[i].VolumeMultiplier = MusicTracks[i].VolumeMultiplier - FadeFactor * DeltaTime;
			if (MusicTracks[i].VolumeMultiplier <= 0.f) {
				//`log("fading out in tick");
				MusicTracks[i].VolumeMultiplier = 0.f;
				MusicTracks[i].Stop();
			}
		}
	}

	NumBeats = (WorldInfo.TimeSeconds - MusicStartTime) * Tempo / 60;
	if (NumBeats - LastBeat < 1)
		return;

	LastBeat = int(NumBeats);
	if (LastBeat % 2 != 0)
		return;

	// process any outstanding pending events
	if (PendingEvent > -2 && WorldInfo.TimeSeconds > PendingEventPlayTime) {
		ProcessMusicEvent();
		PendingEvent = -2;
		return;
	}

	ConfirmedNewState = MST_Ambient;
	foreach LocalPlayerControllers(class'UTPlayerController', PC) {
		PRI = PC.PlayerReplicationInfo;
		NewState = MST_Ambient;

		//Make sure the action track keeps going if the player is currently firing
		//Also kept up to speed via Trace/ProjectileWarning, StartFire and NotifyTakeHit in PC
		if (PC.Pawn != None && PC.Pawn.IsFiring())
			LastActionEventTime = WorldInfo.TimeSeconds;

		//Dial V for victory!
		if (PC.Pawn != None && PawnPoweredUp(PC.Pawn))
			NewState = MST_Victory;
		else if (PRI.bHasFlag)
			NewState = MST_Victory;
		//See if we should just force action
		else if (MinActionTime == -1)
			NewState = MST_Action;
		//Otherwise, immediately wind down to something else (or wind down after MinActionTime from MST_Action)
		else if (CurrentState != MST_Action || WorldInfo.TimeSeconds - LastActionEventTime > MinActionTime) {
			if (PRI.Team != None && UTOnslaughtGame(WorldInfo.Game) != None) {
				bOurCorePowered = UTOnslaughtGame(WorldInfo.Game).PowerCore[PRI.Team.TeamIndex].PoweredBy(1 - PRI.Team.TeamIndex);
				bTheirCorePowered = UTOnslaughtGame(WorldInfo.Game).PowerCore[1 - PRI.Team.TeamIndex].PoweredBy(PRI.Team.TeamIndex);
			}

			//Check flags, flipping the check if it's Warfare
			if (!UTGameReplicationInfo(WorldInfo.GRI).FlagsAreHome()) {
				NewState = MST_Tension;
				if (PRI.Team != None
				&& !UTGameReplicationInfo(WorldInfo.GRI).FlagIsHome((UTOnslaughtGame(WorldInfo.Game) != None) ? 1 - PRI.Team.TeamIndex : PRI.Team.TeamIndex))
					NewState = MST_Suspense;
			}
			//New! Check Power Core status in Warfare
			else if (bOurCorePowered || bTheirCorePowered) {
				NewState = MST_Tension;
				if (bOurCorePowered)
					NewState = MST_Suspense;
			}
		}

		//See if music should otherwise change if it wasn't set above
		if (NewState == MST_Ambient) {
			if (MaxActionTime > 0 && CurrentState >= MST_Action && WorldInfo.TimeSeconds - LastActionEventTime < MaxActionTime)
				NewState = MST_Action;
			//New! Wind down to Tension/Suspense state based on score
			else if (MaxTensionTime > 0 && CurrentState > MST_Ambient && WorldInfo.TimeSeconds - LastActionEventTime < MaxTensionTime + MaxActionTime) {
				NewState = MST_Tension;
				if (PRI.Team != None) {
					foreach WorldInfo.GRI.Teams(TI) {
						if (PRI.Team.Score < TI.Score) {
							NewState = MST_Suspense;
							break;
						}
					}
				}
				else {
					foreach WorldInfo.GRI.PRIArray(PRI) {
						if (PC.PlayerReplicationInfo.Score < PRI.Score) {
							NewState = MST_Suspense;
							break;
						}
					}
				}
			}
		}

		//Only accept the new state if it's above new state set by another local player
		if (NewState > ConfirmedNewState)
			ConfirmedNewState = NewState;
	}

	//We don't want to change track multiple times per tick, so only possibly change tracks here
	if (ConfirmedNewState != CurrentState)
		SetTrack(ConfirmedNewState);
}

//fox: Music should never stop, so restart the track if it's non-looping
function MusicFinished(AudioComponent AC)
{
	if (CurrentTrack == AC)
		SetTrack(CurrentState, true);
}

//fox: Set the new Tempo/FadeFactor variables since the original ones were private in UTMusicManager
function ChangeTrack(EMusicState NewState)
{
	SetTrack(NewState);
}
function SetTrack(EMusicState NewState, optional bool bForceNewTrack = false)
{
	local UTMapInfo UMI;
	local AudioComponent NewTrack;
	local MusicSegment NewSegment;

	//`log("MusicManager: ChangeTrack: " $ NewState);

	//Allow switching from Intro track
	if (CurrentState == NewState && !bForceNewTrack && CurrentTrack != None && CurrentTrack.bWasPlaying) {
		//`log("MusicManager: ChangeTrack: new and current state are the same");
		return;
	}

	CurrentState = NewState;

	UMI = UTMapInfo(WorldInfo.GetMapInfo());
	// means the content folks have not set the map up with music yet
	if (UMI == None) {
		//`log(GetFuncName() @ "UMI is None");
		return;
	}

	// select appropriate track
	switch (NewState) {
		case MST_Ambient:
			NewSegment = UMI.MapMusicInfo.MapMusic.Ambient; break;
		case MST_Tension:
			NewSegment = UMI.MapMusicInfo.MapMusic.Tension; break;
		case MST_Suspense:
			NewSegment = UMI.MapMusicInfo.MapMusic.Suspense; break;
		case MST_Action:
			NewSegment = UMI.MapMusicInfo.MapMusic.Action; break;
		case MST_Victory:
			NewSegment = UMI.MapMusicInfo.MapMusic.Victory; break;
		default: //MST_Max
			NewSegment = UMI.MapMusicInfo.MapMusic.Intro;
	}
	if (MusicTracks[NewState] == None) {
		if (CurrentTrack != None && CurrentTrack.SoundCue == NewSegment.TheCue)
			MusicTracks[NewState] = CurrentTrack;
		else
			MusicTracks[NewState] = CreateNewTrack(NewSegment.TheCue);
	}
	NewTrack = MusicTracks[NewState];
	Tempo = UMI.MapMusicInfo.MapMusic.Tempo;
	FadeFactor = MusicVolume / NewSegment.CrossfadeToMeNumMeasuresDuration;

	if (CurrentTrack == NewTrack && !bForceNewTrack && CurrentTrack != None && CurrentTrack.bWasPlaying)
		return;

	// play selected track
	MusicStartTime = WorldInfo.TimeSeconds;
	LastBeat = 0;
	if (NewTrack != None) {
		NewTrack.OnAudioFinished = MusicFinished; //Anything played here should be infinite duration, so work around this if needed
		NewTrack.SoundCue.SoundGroup = 'Music'; //To fix the bug where some SoundCues (like CBP3-SpaceIt music) used for music are not properly Music group, and so are unaffected by Music volume slider
		NewTrack.VolumeMultiplier = (bForceNewTrack) ? MusicVolume : 0.0; //Don't fade in if forcing new track (like from Intro etc.)
		NewTrack.Play();
		//`Log("foxLog:" @ NewTrack.SoundCue @ NewTrack.SoundCue.SoundGroup @ NewTrack.SoundCue.VolumeMultiplier @ NewTrack.SoundCue.Duration @ NewTrack.SoundCue.GetCueDuration());
	}
	CurrentTrack = NewTrack;
}

defaultproperties
{
	PendingEvent=-2
	MinActionTime=15.0
	MaxActionTime=30.0
	MaxTensionTime=30.0
}
