/**
 * A modified ProfileSettings to save new keybindings ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxProfileSettings extends UTProfileSettings
	config(Game);

var config bool bAllowSplitscreenPlayerKeyboardBinds;

//fox: Overwrite this to ignore EDigitalButtonActions's DBA_MAX, using a custom instead
function ApplyAllKeyBindings(PlayerInput PInput)
{
	local int BindingIdx;

	if (!class'UIRoot'.static.IsConsole()) {
		RemoveDBABindings(PInput);
		for (BindingIdx = 0; BindingIdx < DigitalButtonActionsToCommandMapping.length; BindingIdx++) //For KeyActions1-49, with new ones starting at 45 ~fox
			ApplyKeyBinding(PInput, EDigitalButtonActions(BindingIdx));
	}

	PatchDefaultKeyBindings(PInput);
}

//fox: Overwrite this to not accept keyboard bindings on splitscreen players unless bAllowSplitscreenPlayerKeyboardBinds is true
function ApplyKeyBinding(PlayerInput PInput, EDigitalButtonActions KeyBinding)
{
	local int KeyBindingValue;
	local int CurrentProfileValue;
	local name KeyName;
	local string KeyCommand;
	local int BindingIdx;

	if (GetProfileSettingValueInt(GetProfileIDForDBA(KeyBinding), CurrentProfileValue)) {
		//Unpack value
		for(BindingIdx = 0; BindingIdx < 4; BindingIdx++) {
			KeyBindingValue = 0xFF & (CurrentProfileValue >> (BindingIdx * 8));
			KeyName = FindKeyName(EUTBindableKeys(KeyBindingValue));

			//If we're not going to allow keyboard bindings on a splitscreen player, interdict here ~fox
			if (EUTBindableKeys(KeyBindingValue) < UTBND_LeftStickX && !bAllowSplitscreenPlayerKeyboardBinds && LocalPlayer(PInput.Outer.Player).ControllerId > 0)
				continue;

			KeyCommand = DigitalButtonActionsToCommandMapping[KeyBinding];

			//Unbind the current value first
			if (KeyBindingValue != 0 && KeyName != '' && Len(KeyCommand) > 0) {
				//`Log("### - Applying keybinding Key:"@KeyName@"Command:"@KeyCommand);
				PInput.SetBind(KeyName, KeyCommand);
			}
		}
	}
}

defaultproperties
{
	bAllowSplitscreenPlayerKeyboardBinds=False

	//fox: A total hack, but this works for now
	DigitalButtonActionsToCommandMapping(DBA_ShrinkHUD)="GBA_BasePathRed"
	DigitalButtonActionsToCommandMapping(DBA_GrowHUD)="GBA_BasePathBlue"
	DigitalButtonActionsToCommandMapping(DBA_BestWeapon)="GBA_NetStat"
	DigitalButtonActionsToCommandMapping(DBA_TurnLeft)="GBA_Screenshot"
	DigitalButtonActionsToCommandMapping(DBA_TurnRight)="GBA_FineAim"
	DigitalButtonActionsToCommandMapping(45)="GBA_DropFlag"

	DefaultSettings.Empty

	//Online Service - Defaults for the values if not specified by the online service
	DefaultSettings.Add((Owner=OPPO_OnlineService,ProfileSetting=(PropertyId=PSI_ControllerVibration,Data=(Type=SDT_Int32,Value1=PCVTO_On))))
	DefaultSettings.Add((Owner=OPPO_OnlineService,ProfileSetting=(PropertyId=PSI_YInversion,Data=(Type=SDT_Int32,Value1=PYIO_Off))))
	DefaultSettings.Add((Owner=OPPO_OnlineService,ProfileSetting=(PropertyId=PSI_VoiceMuted,Data=(Type=SDT_Int32,Value1=PYIO_Off))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ControllerSensitivityMultiplier,Data=(Type=SDT_Int32,Value1=12))))
	DefaultSettings.Add((Owner=OPPO_OnlineService,ProfileSetting=(PropertyId=PSI_AutoAim,Data=(Type=SDT_Int32,Value1=PAAO_Off))))

	//UT Specific Defaults
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_CustomCharString,Data=(Type=SDT_String))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_UnlockedCharacters,Data=(Type=SDT_Int32))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_FirstTimeMultiplayer,Data=(Type=SDT_Int32))))

	//Audio
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_SFXVolume,Data=(Type=SDT_Int32,Value1=5))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MusicVolume,Data=(Type=SDT_Int32,Value1=5))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_VoiceVolume,Data=(Type=SDT_Int32,Value1=6))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AmbianceVolume,Data=(Type=SDT_Int32,Value1=5))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AnnouncerVolume,Data=(Type=SDT_Int32,Value1=5))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AnnounceSetting,Data=(Type=SDT_Int32,Value1=ANNOUNCE_FULL))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AutoTaunt,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MessageBeep,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_TextToSpeechMode,Data=(Type=SDT_Int32,Value1=TTSM_None))))

	//Video
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_Gamma,Data=(Type=SDT_Int32,Value1=6))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_PostProcessPreset,Data=(Type=SDT_Int32,Value1=PPP_Default))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ScreenResolutionX,Data=(Type=SDT_Int32,Value1=1024))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ScreenResolutionY,Data=(Type=SDT_Int32,Value1=768))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_DefaultFOV,Data=(Type=SDT_Int32,Value1=75))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_Subtitles,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))

	//Game
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ViewBob,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GoreLevel,Data=(Type=SDT_Int32,Value1=GORE_NORMAL))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_DodgingEnabled,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_WeaponSwitchOnPickup,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_NetworkConnection,Data=(Type=SDT_Int32,Value1=NETWORKTYPE_Cable))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_DynamicNetspeed,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_SpeechRecognition,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_Alias,Data=(Type=SDT_String))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ClanTag,Data=(Type=SDT_String))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ServerDescription,Data=(Type=SDT_String))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AllowCustomCharacters,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AlwaysLoadCustomCharacters,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))

	//Input
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MouseSmoothing,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ReduceMouseLag,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_EnableJoystick,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MouseSensitivityGame,Data=(Type=SDT_Int32,Value1=2500))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MouseSensitivityMenus,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MouseSmoothingStrength,Data=(Type=SDT_Int32,Value1=10))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_MouseAccelTreshold,Data=(Type=SDT_Int32,Value1=10))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_DodgeDoubleClickTime,Data=(Type=SDT_Int32,Value1=25))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_TurningAccelerationFactor,Data=(Type=SDT_Int32,Value1=5))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_ButtonA,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_ButtonB,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_ButtonX,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_ButtonY,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_Back,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_RightBumper,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_LeftBumper,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_RightTrigger,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_LeftTrigger,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_RightThumbstickPressed,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_LeftThumbstickPressed,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_DPadUp,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_DPadDown,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_DPadLeft,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_DPadRight,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_GamepadBinding_AnalogStickPreset,Data=(Type=SDT_Int32,Value1=ESA_Normal))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AutoCenterPitch,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AutoCenterVehiclePitch,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_TiltSensing,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_EnableHardwarePhysics,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_VehicleControls,Data=(Type=SDT_Int32,Value1=UTVC_Simple))))


	//Weapons
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_WeaponHand,Data=(Type=SDT_Int32,Value1=HAND_Right))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_SmallWeapons,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_DisplayWeaponBar,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowOnlyAvailableWeapons,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_yES))))


	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_RocketLauncherPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_BioRiflePriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_FlakCannonPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_SniperRiflePriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_LinkGunPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_EnforcerPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShockRiflePriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_StingerPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_AVRILPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_RedeemerPriority,Data=(Type=SDT_Float))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_CrosshairType,Data=(Type=SDT_Int32,Value1=CHT_Normal))))

	//HUD
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowMap,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowClock,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowDoll,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowAmmo,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowPowerups,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowScoring,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowLeaderboard,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_RotateMap,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_ShowVehicleArmorCount,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_NO))))

	//Onslaught/Warfare
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_PopupMapOnDeath,Data=(Type=SDT_Int32,Value1=UTPID_VALUE_YES))))

	//PC Keys
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_1,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_2,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_3,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_4,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_5,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_6,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_7,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_8,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_9,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_10,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_11,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_12,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_13,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_14,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_15,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_16,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_17,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_18,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_19,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_20,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_21,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_22,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_23,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_24,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_25,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_26,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_27,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_28,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_29,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_30,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_31,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_32,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_33,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_34,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_35,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_36,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_37,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_38,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_39,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_40,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_41,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_42,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_43,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_44,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_45,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_46,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_47,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_48,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=UTPID_KeyAction_49,Data=(Type=SDT_Int32,Value1=0))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_SinglePlayerMapMaskA,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_SinglePlayerMapMaskB,Data=(Type=SDT_Int32,Value1=0))))

	//Single Player
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot0,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot1,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot2,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot3,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot4,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot5,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot6,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot7,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot8,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot9,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot10,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot11,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot12,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot13,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot14,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot15,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot16,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot17,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot18,Data=(Type=SDT_Int32,Value1=0))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_PersistentKeySlot19,Data=(Type=SDT_Int32,Value1=0))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_SinglePlayerSkillLevel,Data=(Type=SDT_Int32,Value1=1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_SinglePlayerCurrentMission,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_SinglePlayerCurrentMissionResult,Data=(Type=SDT_Int32,Value1=0))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck0,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck1,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck2,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck3,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck4,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck5,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck6,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck7,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck8,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck9,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck10,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck11,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck12,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck13,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck14,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck15,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck16,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck17,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck18,Data=(Type=SDT_Int32,Value1=-1))))
	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ModifierCardDeck19,Data=(Type=SDT_Int32,Value1=-1))))

	DefaultSettings.Add((Owner=OPPO_Game,ProfileSetting=(PropertyId=PSI_ChapterMask,Data=(Type=SDT_Int32,Value1=0))))
}
