/**
 * A mutator that replaces various SquadAIs with foxSquadAIs ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class UTMutator_foxAI extends UTMutator
	config(Mutator_foxAI);

var array<string> ActivationErrors;
var class<Object> OldTeamAIType;
var class<Object> OldBotClass;
var class<Object> OldPlayerReplicationInfoClass;
var bool bMultiTeamSupportRequired;

//Hack to determine if we should treat tracked turrets as AIP_Offensive and eligible for flag running etc.
var bool bRailsHack;
//Hack to determine if we should treat tanks/darkwalkers as bIsOnTrack for "trapped" vehicles like the Darkwalker on Containment
var bool bContainmentHack;
//Cache of current level's UTTrackTurretPathNodes, used in various TrackTurret logic across multiple classes
//Note: Here instead of AvoidanceMapper as that might not be active per user preference
var array<UTTrackTurretPathNode> TrackTurretNodes;

var foxAvoidanceMapper AvoidanceMapper;
var foxVoiceManager VoiceManager;

var globalconfig bool bForceActivate;
var globalconfig bool bSkipActivationErrorBroadcast;
var globalconfig bool bExtendedVoices;

function InitMutator(string Options, out string ErrorMessage)
{
	local UTTeamGame TG;
	local UTGame UTG;
	local Actor A;

	Super.InitMutator(Options, ErrorMessage);

	//Team Game TeamAI Overrides (sets SquadAI and randomized orders)
	TG = UTTeamGame(WorldInfo.Game);
	if (TG != None) {
		OldTeamAIType = TG.TeamAIType[0];
		bMultiTeamSupportRequired = TG.DefaultEnemyRosterClass == "UTMultiTeam.UTMT_TeamInfo";
		if (TG.TeamAIType[0] == class'UTCTFTeamAI') {
			TG.TeamAIType[0] = class'foxCTFTeamAI';
			TG.TeamAIType[1] = class'foxCTFTeamAI';
		}
		else if (TG.TeamAIType[0] == class'UTOnslaughtTeamAI') {
			TG.TeamAIType[0] = class'foxOnslaughtTeamAI';
			TG.TeamAIType[1] = class'foxOnslaughtTeamAI';
		}
		else if (TG.TeamAIType[0] == class'UTGreedTeamAI') {
			TG.TeamAIType[0] = class'foxGreedTeamAI';
			TG.TeamAIType[1] = class'foxGreedTeamAI';
		}
		else if (bForceActivate || TG.TeamAIType[0] == class'UTTeamAI') {
			TG.TeamAIType[0] = class'foxTeamAI';
			TG.TeamAIType[1] = class'foxTeamAI';
		}
	}

	//Other Overrides
	UTG = UTGame(WorldInfo.Game);
	if (UTG != None) {
		//DM Game Roster Override (sets SquadAI)
		if (bForceActivate || UTG.DefaultEnemyRosterClass == "UTGame.UTDMRoster")
			UTG.DefaultEnemyRosterClass = "foxMod_AI.foxDMRoster";
		//UTBot Override (misc AI stuff)
		OldBotClass = UTG.BotClass;
		if (bForceActivate || UTG.BotClass == class'UTBot')
			UTG.BotClass = class'foxBot';

		//PRI Overrides (for extended voice support and some AI stuff)
		OldPlayerReplicationInfoClass = UTG.PlayerReplicationInfoClass;
		switch UTG.PlayerReplicationInfoClass {
			case class'UTBetrayalPRI':
				UTG.PlayerReplicationInfoClass = class'foxBetrayalPRI'; break;
			case class'UTDuelPRI':
				UTG.PlayerReplicationInfoClass = class'foxDuelPRI'; break;
			case class'UTGreedPRI':
				UTG.PlayerReplicationInfoClass = class'foxGreedPRI'; break;
			case class'UTHeroPRI':
				UTG.PlayerReplicationInfoClass = class'foxHeroPRI'; break;
			case class'UTMissionSelectionPRI':
				UTG.PlayerReplicationInfoClass = class'foxMissionSelectionPRI'; break;
			case class'UTOnslaughtHeroPRI':
				UTG.PlayerReplicationInfoClass = class'foxOnslaughtHeroPRI'; break;
			case class'UTOnslaughtPRI':
				UTG.PlayerReplicationInfoClass = class'foxOnslaughtPRI'; break;
			case class'UTPlayerReplicationInfo':
				UTG.PlayerReplicationInfoClass = class'foxPlayerReplicationInfo'; break;
			default:
				if (bForceActivate)
					UTG.PlayerReplicationInfoClass = class'foxPlayerReplicationInfo';
		}
	}

	//Hack for VCTF-Rails -- TODO better name and move this to configurable list
	bRailsHack = Left(WorldInfo.GetMapName(), 7) ~= "RAILS";

	//Hack for VCTF-Containment -- TODO better name and move this to configurable list
	bContainmentHack = Left(WorldInfo.GetMapName(), 13) ~= "CONTAINMENT";

	//Run DoActorTweaks on all existing actors to catch ones that might not be caught otherwise (e.g. map-placed actors)
	TrackTurretNodes.length = 0;
	foreach WorldInfo.AllActors(class'Actor', A)
		DoActorTweaks(A);

	SetTimer(5.0, false, 'CheckErrors');
}

//fox: Check any activation errors
function CheckErrors()
{
	local UTGame UTG;
	local UTTeamGame TG;

	UTG = UTGame(WorldInfo.Game);
	if (UTG == None) {
		ThrowError("GameInfo missing! Something is", WorldInfo.Game, "Everything");
		return;
	}

	if (UTG.BotClass == OldBotClass)
		ThrowError("BotClass", UTG.BotClass, "Individual bot AI");
	if (UTG.PlayerReplicationInfoClass == OldPlayerReplicationInfoClass)
		ThrowError("PlayerReplicationInfoClass", UTG.PlayerReplicationInfoClass, "Advanced AI and voice fixes");

	TG = UTTeamGame(WorldInfo.Game); //If bMultiTeamSupportRequired we can assume TeamAIType is not used, so throw an error
	if (TG != None && (TG.TeamAIType[0] == OldTeamAIType || bMultiTeamSupportRequired))
		ThrowError("TeamAI", TG.TeamAIType[0], "Gametype AI");
	if (UTG.DefaultEnemyRosterClass == "UTGame.UTDMRoster")
		ThrowError("DMRoster", UTG.DefaultEnemyRosterClass, "Gametype AI");

	SetTimer(1.0, false, 'BroadcastErrors');
}
function ThrowError(string ClassName, coerce string Reason, string BrokenFunctions)
{
	local string Error;

	Error = "foxMod AI:" @ ClassName @ "incompatible with" @ Reason $ ".";
	ActivationErrors.AddItem(Error);
	`Log(Error);

	Error = "        --" @ BrokenFunctions @ "will not function.";
	ActivationErrors.AddItem(Error);
	`Log(Error);
}
function BroadcastErrors()
{
	local PlayerController C;
	local string E;

	if (bSkipActivationErrorBroadcast || ActivationErrors.Length < 1)
		return;

	foreach WorldInfo.AllControllers(class'PlayerController', C) {
		foreach ActivationErrors(E)
			C.ClientMessage(E);
		C.ClientMessage("foxMod AI: One or more errors have occurred. Please check the console (~ key) for more info.");
	}
}

//fox: Fetch the active Avoidance Mapper (or spawn one if it doesn't exist)
function foxAvoidanceMapper GetAvoidanceMapper()
{
	if (AvoidanceMapper == None)
		foreach WorldInfo.DynamicActors(class'foxAvoidanceMapper', AvoidanceMapper)
			break;
	if (AvoidanceMapper == None)
		AvoidanceMapper = Spawn(class'foxAvoidanceMapper');
	return AvoidanceMapper;
}

//fox: Fetch the active Voice Manager (or spawn one if it doesn't exist)
function foxVoiceManager GetVoiceManager()
{
	if (VoiceManager == None)
		foreach WorldInfo.DynamicActors(class'foxVoiceManager', VoiceManager)
			break;
	if (VoiceManager == None)
		VoiceManager = Spawn(class'foxVoiceManager');
	return VoiceManager;
}

//fox: Fix up DefensePriority for Onslaught objectives
static function FixDefensePriorityFor(UTGameObjective O)
{
	local UTOnslaughtNodeObjective Node;
	local UTOnslaughtCountdownNode CountdownNode;
	local UTOnslaughtPowerCore Core;
	local int DefensePriority;

	//Force DefensePriority for UTOnslaughtObjectives as they change to undesirable values during gameplay
	if (UTOnslaughtObjective(O) == None)
		return;
	DefensePriority = O.default.DefensePriority;

	//Set priority for NodeObjectives based on usefulness
	Node = UTOnslaughtNodeObjective(O);
	if (Node != None) {
		DefensePriority += Node.ActivatedObjectives.Length * 2;
		DefensePriority += Node.VehicleFactories.Length;
		DefensePriority += Node.NodeTeleporters.Length / 2;
		DefensePriority += Node.DeployableLockers.Length / 2;
		if (Node.FlagBase != None)
			DefensePriority += 2;
		if (Node.bStandalone)
			DefensePriority -= 1;
	}

	//Set priority for CountdownNodes based on total time and if node damages core
	CountdownNode = UTOnslaughtCountdownNode(O);
	if (CountdownNode != None) {
		DefensePriority *= CountdownNode.CountdownTime / float(CountdownNode.default.CountdownTime);
		if (CountdownNode.bDamagesCore)
			DefensePriority *= 4;
	}

	//Slightly prefer SpecialObjectives (most e.g. Barricades are 0 anyways)
	if (UTOnslaughtSpecialObjective(O) != None)
		DefensePriority += 1;

	//Always hit the powercore! (Vanilla objective logic handles bNeverAttack cores properly)
	Core = UTOnslaughtPowerCore(O);
	if (Core != None)
		DefensePriority = 255;

	//Apply the new DefensePriority
	O.DefensePriority = byte(Min(DefensePriority, 255));
}

//fox: Change various properties of things (CheckActorTweaks also called from PostBeginPlay)
function bool CheckReplacement(Actor Other)
{
	//Check replacement first - if we're being replaced, then forget it, hopefully we'll catch the new object later
	if (!Super.CheckReplacement(Other))
		return false;

	DoActorTweaks(Other);
	return true;
}
function DoActorTweaks(Actor Other)
{
	local UTVehicle V;
	local NavigationPoint N;
	local WaterVolume WV;
	local UTGameObjective O, BestO;
	local float Dist, BestDist;

	//If it's a UTTrackTurretPathNode, add it to the list
	if (UTTrackTurretPathNode(Other) != None) {
		TrackTurretNodes.AddItem(UTTrackTurretPathNode(Other));
		//`Log("foxLog:" @ Self @ "TrackTurretNodes:" @ TrackTurretNodes.length);
	}

	//Force alternate locations for all actors
	if (class'foxBot'.default.bForceAlternateTargetLocations)
		Other.bHasAlternateTargetLocation = true;

	//HACK never bWaitUntilCompletelyOpened as DoorMarker code is bugged :(
	if (DoorMarker(Other) != None)
		DoorMarker(Other).bWaitUntilCompletelyOpened = false;

	//Allow extended vehicle pathing
	if (class'foxBot'.default.ExtendedVehiclePathingLevel > -1) {
		if (Pawn(Other) != None) {
			Pawn(Other).bPathfindsAsVehicle = false;
			Pawn(Other).NonPreferredVehiclePathMultiplier = 1.0;
		}
		if (UTVehicle(Other) != None)
			UTVehicle(Other).bUseAlternatePaths = false; //Let avoidance mapping handle alternate routes for vehicles - vanilla detours are painful
	}

	//Avoid NavigationPoints submerged in water until we can figure out better swimming
	N = NavigationPoint(Other);
	if (N != None && class'foxBot'.default.bAvoidSwimming) {
		foreach WorldInfo.AllActors(class'WaterVolume', WV) {
			if (WV.Encompasses(N)) {
				//`Log("foxLog: Adding ADDCOSTWATER to WaterVolume NavigationPoint" @ N);
				N.ExtraCost += class'foxBot'.const.ADDCOSTWATER;
				break;
			}
		}
	}

	//Make sure DefensePoints are always hooked up to an objective (just assume closest one)
	if (UTDefensePoint(Other) != None
	&& UTDefensePoint(Other).DefendedObjective == None) {
		foreach WorldInfo.AllActors(class'UTGameObjective', O) {
			Dist = VSize(O.Location - Other.Location);
			if (FastTrace(O.Location, Other.Location))
				Dist *= 0.5;
			if (BestO == None || Dist < BestDist) {
				BestO = O;
				BestDist = Dist;
			}
		}
		if (BestO != None) {
			//`Log("foxLog:" @ Other @ " has no DefendedObjective, assuming" @ BestO);
			UTDefensePoint(Other).DefendedObjective = BestO;
		}
	}

	//Always use default DefensePriority and don't auto-adjust for core-linked powernodes, standalone nodes, etc.
	//Also in foxOnslaughtSquadAI (and not just here) to catch DefensePriorities changing dynamically from UTOnslaughtNodeObjective.CheckLink() etc. -- this should match foxOnslaughtSquadAI
	if (UTOnslaughtObjective(Other) != None)
		FixDefensePriorityFor(UTOnslaughtObjective(Other));

	//Handle vehicle-related hacks
	V = UTVehicle(Other);
	if (V != None) {
		//Treat Goliath and DarkWalker as offensive vehicles, not any-purpose
		if (UTVehicle_Goliath(V) != None || UTVehicle_DarkWalker(V) != None)
			V.AIPurpose = AIP_Offensive;

		//Treat bIsOnTrack vehicles as defensive vehicles, so attackers don't sit around idling in them
		//Note: They'll still use them to kill enemies as per the RoleAppropriateVehicleFor enemy check in SquadAI
		if (V.bIsOnTrack)
			V.AIPurpose = AIP_Defensive;

		//Allow Scavenger to jump when appropriate
		if (UTVehicle_Scavenger(V) != None)
			V.bScriptedRise = true;

		//Modify this vehicle if we're using the Rails or Containment hacks
		if (bRailsHack && V.bIsOnTrack)
			V.AIPurpose = AIP_Offensive; //We don't want defenders idling in these and taking off toward the enemy base
		if (bContainmentHack && UTVehicle_DarkWalker(V) != None)
			V.bIsOnTrack = true;
	}

	//BioRifle doesn't have ShouldFireOnRelease set when it, well, should
	if (UTWeap_BioRifle_Content(Other) != None)
		UTWeap_BioRifle_Content(Other).ShouldFireOnRelease[1] = 1;
	//Whereas ShockRifle does have it set and I can't see why as it's not needed for AI comboing
	else if (UTWeap_ShockRifle(Other) != None)
		UTWeap_ShockRifle(Other).ShouldFireOnRelease[1] = 0;

	//Some weapons (Impact Hammer) actually have more than two fire modes (for internal reasons or otherwise), so extend ShouldFireOnRelease to match
	//This could be fixed in UTPawn with a Length check but then it would be dependent on foxMod base
	if (UTWeapon(Other) != None) {
		UTWeapon(Other).ShouldFireOnRelease.Length = UTWeapon(Other).WeaponFireTypes.Length;
		UTWeapon(Other).bSplashJump = false; //Also stop jumping like an idiot
	}
}

defaultproperties
{
	GroupNames[0]="FOXMODAI"
	bForceActivate=False
	bSkipActivationErrorBroadcast=False
	bExtendedVoices=True
}
