/**
 * Contains UTBot function overrides to mainly try and fix issues with bots getting stuck ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxBot extends UTBot
	config(Mutator_foxAI);

/*
//fox: For reference, since this is not documented within UScript
enum EReachSpecFlags
{
	R_WALK = 1,	//walking required
	R_FLY = 2,	//flying required
	R_SWIM = 4,	//swimming required
	R_JUMP = 8,	//jumping required
	R_DOOR = 16,
	R_SPECIAL = 32,
	R_LADDER = 64,
	R_PROSCRIBED = 128,
	R_FORCED = 256,
	R_PLAYERONLY = 512
};
*/

//fox: Various constants used - these may become configurable variables later
const MELEERANGE = 1024.0;
const ADDCOSTBLOCKED = 50000.0;
const ADDCOSTENEMY = 32000.0; //Divided by number of combatants, using 8 as base
const ADDCOSTWATER = 4000.0;
const ADDCOSTLIFTWAIT = 10000.0;
const ADDCOSTSQUADMOVE = 1000.0;

//fox: Whether to force alternate target locations (for Mutator)
var config bool bForceAlternateTargetLocations;

//fox: How to use extended vehicle pathing (for Mutator: 1 = use extended vehicle pathing, 0 = use extended vehicle pathing but don't validate vehicle paths (faster), -1 = disabled)
var config int ExtendedVehiclePathingLevel;

//fox: Points that we chould avoid based on collision size, physics volume if in vehicle, etc.
var array<NavigationPoint> InvalidNodes;
var bool bUsingParkingSpots;
var bool bReachedParkingSpot;
var bool bPathingFailed;

//fox: Locations over time from previous CheckIfStuck calls, with lower indices being more recent
var array<NavigationPoint> UnreachableNodes;
var Vector OldLocation[3];
var float LastCheckIfStuckTime;
var float LastUnstuckTime;
var config float CheckIfStuckInterval;
var config float CheckIfStuckRadius;

//fox: Current UnstuckTarget, cleared every UnstuckTargetInterval seconds
var Actor UnstuckTarget;
var float UnstuckTargetTime;
var config float UnstuckTargetInterval;
var config float UnstuckTargetMinRadius;
var config float UnstuckNudgeVelocity;

//fox: Array of weapons, in order, that we should use to shoot door targets
var DoorMarker DoorTarget;
var array<class<Inventory> > PreferredDoorWeapons;

//fox: Weapon switching rate limit
var float LastWeaponSwitchTime;

//fox: Special melee combat behavior
var bool bDisruptShieldBelts;
var float DisruptShieldBeltsCheckTime;

//fox: Next allowed TranslocateHack time (as NextTranslocTime is modified by UTProj_TransDisc)
var float NextTranslocHackTime;

//fox: Current Squad FormationSize (set from foxSquadAIs, used for MayDodgeToMoveTarget)
var float CurrentFormationSize;

//fox: Gather vehicle passengers behavior
var bool bGatherVehiclePassengers;

//fox: Swimming location adjustment fix
var Vector SwimmingFixOldLocation;
var config bool bSwimmingHitWallFix;
var config bool bAvoidSwimming;

//fox: Cached DefensePoint to reset a Hold order after death
var UTDefensePoint CachedDefensePoint;

//fox: Configurable max sight radius
var config float MaxSightRadius;

//fox: Configurable min inventory weights
var config float MinForceNeedWeaponWeight;
var config float MinForceWantInventoryWeight;

//fox: Automatic flag and weapon tossing
var float LastWeaponTossTime;
var config float WeaponTossInterval;
var config float FlagTossInterval;
var config float MaxTossDist;
var config float TossVelocityMult;

//fox: Avoidance Mapping - expensive! (AvoidanceMappingLevel: 2 = always, 1 = only when heading to objective, 0 = only flag/orb carriers, -1 = disabled)
var float LastForceRefreshRouteTime;
var float ForceRefreshRouteInterval;
var foxAvoidanceMapper AvoidanceMapper;
var config int AvoidanceMappingLevel;
var config bool bAvoidanceMappingUsePlayerCount;
var config bool bAvoidanceMappingAvoidTeammates;
var config bool bAvoidanceMappingUpdateVehicles;
var config float AvoidanceMappingBuildInterval;
var config float AvoidanceMappingSearchRadius;
var config bool bAvoidanceMappingFullTrace;

//fox: Accumulate FearCost, and how much to scale FearCost per death
var config int DeathFearCostScaleFactor;
var config bool bDeathFearCostAccumulate;

//fox: Time we entered vehicle (used to ensure we don't get out too quickly)
var float VehicleEnterTime;
var config float MinVehicleTime;
var config float MinUnDeployTime;

//fox: For deploy hack in WhatToDoNext and CanAttack
var UTVehicle_Deployable CachedDeployableVehicle;

//fox: Max orders of type that can be handed out by random orders (-1 = unlimited)
var config int RandomOrdersMaxDefenders;
var config int RandomOrdersMaxAttackers;
var config int RandomOrdersMaxFollowers;

//fox: Min/max time between random order reassignment (-1 = don't randomly reassign orders)
var config float RandomOrdersMinReassignTime;
var config float RandomOrdersMaxReassignTime;

//fox: Squad Settings
var config bool bLoneFreelanceSquads;
var config int MaxSquadSize;
var config int MaxSquadRoutes;

//fox: Easter egg!
var config bool bAlphaAndOmega;

//fox: For delayed auto-taunts
var PlayerReplicationInfo TRecipient;
var name TMessageType;
var class<DamageType> TDamageType;

//fox: Sanity check all config variables
function PostBeginPlay()
{
	Super.PostBeginPlay();

	if (MinVehicleTime != -1)
		MinVehicleTime = FMax(MinVehicleTime, 0);
	if (MinUnDeployTime != -1)
		MinUnDeployTime = FMax(MinUnDeployTime, 0);

	ExtendedVehiclePathingLevel = Clamp(ExtendedVehiclePathingLevel, -1, 1);

	AvoidanceMappingLevel = Clamp(AvoidanceMappingLevel, -1, 2);
	if (AvoidanceMappingSearchRadius != -1)
		AvoidanceMappingSearchRadius = FMax(AvoidanceMappingSearchRadius, 1.0);
	if (AvoidanceMappingBuildInterval != -1)
		AvoidanceMappingBuildInterval = FMax(AvoidanceMappingBuildInterval, 0.000001);

	DeathFearCostScaleFactor = Max(DeathFearCostScaleFactor, 0);

	if (MaxSightRadius != -1)
		MaxSightRadius = FMax(MaxSightRadius, 1.0);

	if (MaxSightRadius != -1)
		MinForceNeedWeaponWeight = FMax(MinForceNeedWeaponWeight, 0);
	if (MaxSightRadius != -1)
		MinForceWantInventoryWeight = FMax(MinForceWantInventoryWeight, 0);

	if (WeaponTossInterval != -1)
		WeaponTossInterval = FMax(WeaponTossInterval, 0.000001);
	if (FlagTossInterval != -1)
		FlagTossInterval = FMax(FlagTossInterval, 0.000001);
	MaxTossDist = FMax(MaxTossDist, 1.0);
	TossVelocityMult = FMax(TossVelocityMult, 0.01);

	if (RandomOrdersMinReassignTime != -1)
		RandomOrdersMinReassignTime = FMax(RandomOrdersMinReassignTime, 0.000001);
	if (RandomOrdersMaxReassignTime != -1)
		RandomOrdersMaxReassignTime = FMax(RandomOrdersMaxReassignTime, RandomOrdersMinReassignTime);

	CheckIfStuckRadius = FMax(CheckIfStuckRadius, 1.0);
	if (CheckIfStuckInterval != -1)
		CheckIfStuckInterval = FMax(CheckIfStuckInterval, 0.000001);

	UnstuckTargetMinRadius = FMax(UnstuckTargetMinRadius, 1.0);
	if (UnstuckTargetInterval != -1)
		UnstuckTargetInterval = FMax(UnstuckTargetInterval, 0.000001);
	if (UnstuckNudgeVelocity != -1)
		UnstuckNudgeVelocity = FMax(UnstuckNudgeVelocity, 0.000001);

	if (MaxSquadSize != -1)
		MaxSquadSize = FMax(MaxSquadSize, 1);
	if (MaxSquadRoutes != -1)
		MaxSquadRoutes = FMax(MaxSquadRoutes, 1);

	SaveConfig();
}

//fox: Pull any needed data from Mutator
function PullMutatorData()
{
	local Mutator M;

	//This must be true at some point as we wouldn't exist without the mutator!
	for (M = WorldInfo.Game.BaseMutator; M != None; M = M.NextMutator)
		if (UTMutator_foxAI(M) != None)
			break;
	if (M == None) {
		`Log("foxLog: PullMutatorData on" @ Self @ "has no Mutator! This should not have happened. Skipping PullMutatorData...");
		return;
	}

	//Set up our AvoidanceMapper (if applicable)
	if (AvoidanceMapper == None
	&& AvoidanceMappingBuildInterval != -1
	&& AvoidanceMappingLevel > -1) {
		AvoidanceMapper = UTMutator_foxAI(M).GetAvoidanceMapper();

		if (AvoidanceMapper != None)
			AvoidanceMapper.Setup(AvoidanceMappingBuildInterval, AvoidanceMappingSearchRadius, bAvoidanceMappingFullTrace);
	}
}

//fox: Random stuff we should do regardless of game mode
function bool AssignSquadResponsibility()
{
	//local NavigationPoint N;
	local Vector MoveDir, PawnDir;
	local UTVehicle V;
	local bool bShouldBoost;
	local float LastLastSearchTime;

	//From UTBot, check it here instead of Super
	if (LastAttractCheck == WorldInfo.TimeSeconds)
		return false;
	LastAttractCheck = WorldInfo.TimeSeconds;

	//If stationary turret / gunning start running timed WhatToDoNexts
	V = UTVehicle(Pawn.GetVehicleBase());
	if (V != None && (V.bStationary || V.bIsOnTrack || V.Controller != Self)) {
		GoalString = "Do turret stuff";
		SetTimer(1.0, false, 'WhatToDoNext');
	}

	//HACK Don't Scorpion Self-Destruct on objectives, as it's done _way_ too early due to Scorpion code error
	if (UTVehicle_Scorpion(Pawn) != None && Vehicle(Focus) == None) {
		UTVehicle_Scorpion(Pawn).bAISelfDestruct = false;
		Pawn.ClearTimer('CheckScriptedSelfDestruct');
	}

	//See if we still have a valid DoorTarget
	CheckDoorTarget();

	//Also check tosses while here
	CheckFlagTransfer();
	CheckWeaponTransfer();

	//Check if we're stuck
	if (CheckIfStuckInterval != -1 && CheckIfStuck()) {
		//Also just PickNewLeader if we're SquadLeader, as we may be here a while
		if (Squad.SquadLeader == Self)
			Squad.PickNewLeader();
		return true;
	}

	//If we're chasing someone with a melee weapon, no fear!
	if (Enemy != None && Focus == Enemy && Pawn.Weapon != None && Pawn.Weapon.bMeleeWeapon
	&& VSize(Enemy.Location - Pawn.Location) < MELEERANGE) {
		FightEnemy(true, -1000.0);
		return true;
	}

	//Always crouch in a DarkWalker if we can't see our focal point
	if (UTVehicle_DarkWalker(Pawn) != None
	&& !FastTrace(FocalPoint, Pawn.Location))
		UTVehicle_DarkWalker(Pawn).SetInputs(0.0, 0.0, -1.0);

	//Smarter Nemesis turret height switching
	if (UTVehicle_Nemesis(Pawn) != None) {
		if (Enemy == None)
			UTVehicle_Nemesis(Pawn).SetTurretHeight(THS_Lowered);
		else if (InRealVehicle(Enemy.Controller) && CanAttack(Enemy))
			UTVehicle_Nemesis(Pawn).SetTurretHeight(THS_Raised);
		else
			UTVehicle_Nemesis(Pawn).CheckAIStance();
	}

	//HACK CITY Use Scorpion / Scavenger boost if we're facing our MoveTarget and it's far away or above us (ramp) -- TODO hook up a config for this
	PawnDir = Normal(Vector(Pawn.Rotation));
	PawnDir.Z = 0.0;
	//TODO this should eventually iterate the RouteCache, but instead we're using MoveTarget for now
	if (MoveTarget != None) {
		MoveDir = Normal(MoveTarget.Location - Pawn.Location);
		MoveDir.Z = 0.0;
		if (PawnDir dot MoveDir > 0.9 && FastTrace(MoveTarget.Location, Pawn.Location)
		&& (MoveTarget.Location.Z > Pawn.Location.Z + 128.0 || VSize(MoveTarget.Location - Pawn.Location) > 512.0))
			bShouldBoost = true;
	}
	//foreach RouteCache(N) {
	//	if (N == None)
	//		continue;
	//	MoveDir = Normal(N.Location - Pawn.Location);
	//	MoveDir.Z = 0.0;
	//	if (PawnDir dot MoveDir > 0.8 && FastTrace(N.Location, Pawn.Location)
	//	&& (N.Location.Z > Pawn.Location.Z + 128.0 || VSize(N.Location - Pawn.Location) > 4096.0)) {
	//		bShouldBoost = true;
	//		break;
	//	}
	//}
	//if (!bShouldBoost)
	//	ClearTimer('WhatToDoNext');
	if (UTVehicle_Scorpion(Pawn) != None
	&& UTVehicle_Scorpion(Pawn).bBoostersActivated != bShouldBoost) {
		//if (bShouldBoost)
		//	SetTimer(0.5, true, 'WhatToDoNext');
		UTVehicle_Scorpion(Pawn).bTryToBoost = bShouldBoost;
	}
	else if (UTVehicle_Scavenger(Pawn) != None
	&& UTVehicle_Scavenger(Pawn).BallStatus.bIsInBallMode != bShouldBoost) {
		//if (bShouldBoost)
		//	SetTimer(0.5, true, 'WhatToDoNext');
		SetTimer(0.01, false, 'TimedUpdateScavengerBallMode');
	}
	else if (UTVehicle_Fury(Pawn) != None && bShouldBoost) {
		//SetTimer(0.5, true, 'WhatToDoNext');
		UTVehicle_Fury(Pawn).ServerBoost(EBD_Forward);
	}

	//Shoot at visible enemy locked vehicles
	if (Skill + Tactics > 3.0 && !PlayerReplicationInfo.bHasFlag
	&& (UTVehicle(Focus) != None || Focus == None || Focus != LastFireTarget)
	&& (Enemy == None || !CanAttack(Enemy))) {
		for (V = UTGame(WorldInfo.Game).VehicleList; V != None; V = V.NextVehicle) {
			if (V.bTeamLocked && V.Health > 40
			&& (Squad.Team == None || V.Team != Squad.Team.TeamIndex)
			&& FastTrace(V.Location, Pawn.Location) && CanAttack(V)) {
				Enemy = V; //Do this instead of DoRangedAttackOn so we keep moving
				break;
			}
		}
	}

	//HACK If we have a BioRifle try to start charging it since we can hold the charge forever
	if (Skill + Tactics > 3.0 && UTWeap_BioRifle_Content(Pawn.Weapon) != None
	&& !Pawn.IsFiring() && (Enemy == None || !CanAttack(Enemy))) {
		bCanFire = true;
		bStoppedFiring = false;
		Pawn.StartFire(1);
	}

	//Always look for weapon if needed
	LastLastSearchTime = LastSearchTime;
	if (MinForceNeedWeaponWeight != -1 && NeedWeapon() && FindInventoryGoal(MinForceNeedWeaponWeight)) {
		GoalString = "Need weapon or ammo" @ RouteGoal;
		NoVehicleGoal = RouteGoal;
		SetAttractionState();
		return true;
	}

	//Always look for additional inventory (weapons, ammo, armor etc.) if desired
	if (MinForceWantInventoryWeight != -1 && FindInventoryGoal(MinForceWantInventoryWeight)) {
		GoalString = "Want inventory" @ RouteGoal;
		NoVehicleGoal = RouteGoal;
		SetAttractionState();
		return true;
	}

	//Actually assign squad responsibility, doing this directly instead of Super because we already did LastAttractCheck
	LastSearchTime = LastLastSearchTime;
	return Squad.AssignSquadResponsibility(Self);
}

//fox: Other random stuff we should do regardless of game mode
protected event ExecuteWhatToDoNext()
{
	//Always avoid ledges and adjust from walls
	Pawn.bAvoidLedges = true;
	bAdjustFromWalls = true;

	Super.ExecuteWhatToDoNext();

	//Handle deploy hack as we may have started 0.01s ServerToggleDeploy timer
	DeployHack();

	//Also make sure we don't randomly switch our focus if overridden
	TimedUpdateFocus();

	//Try a TimedJumpDodgeHack?
	SetTimer(0.1, false, 'TimedJumpDodgeHack');
}

//fox: Hook SetupSpecialPathAbilities to add dynamic costs - also contains AddTransientCosts in SquadAI but in Bot instead to be used for all modes (including DM)
event SetupSpecialPathAbilities()
{
	local NavigationPoint N;
	local Controller C;
	local UTBot S;
	local LiftExit LE;
	local float RelStr;
	local byte i, MapperIndex;

	Super.SetupSpecialPathAbilities();

	//Don't use squad routes
	bUsingSquadRoute = false;

	//Also call AddTransientCosts in Squad if appropriate - moved here for simplicity
	for (S = Squad.SquadMembers; S != None; S = S.NextSquadMember) {
		if (S != Self && NavigationPoint(S.MoveTarget) != None && S.InLatentExecution(S.LATENT_MOVETOWARD)) {
			NavigationPoint(S.MoveTarget).TransientCost += ADDCOSTSQUADMOVE;
			if (UTTranslocatorReachSpec(S.CurrentPath) != None
			&& S.Pawn != None && S.Pawn.Anchor != None && S.Pawn.ValidAnchor())
				S.Pawn.Anchor.TransientCost += ADDCOSTSQUADMOVE;
		}
	}

	//Add blocking cost to any UnreachableNodes (added to by CheckIfStuck)
	foreach UnreachableNodes(N) {
		if (N != None) {
			N.TransientCost += ADDCOSTBLOCKED;
			//`Log("foxLog: Avoiding unreachable node" @ N @ "with cost" @ N.TransientCost);
		}
	}

	//Add blocking cost to any InvalidNodes (added to by CheckInvalidNodes)
	foreach InvalidNodes(N) {
		if (N != None) {
			N.TransientCost += ADDCOSTBLOCKED;
			//`Log("foxLog: Avoiding invalid node" @ N @ "with cost" @ N.TransientCost);
		}
	}

	//Avoidance Mapping checks
	if (AvoidanceMappingLevel < 0 || AvoidanceMapper == None || Pawn == None
	|| (AvoidanceMappingLevel < 1 && !PlayerReplicationInfo.bHasFlag)
	|| (AvoidanceMappingLevel < 2 && Squad.SquadObjective == None) //!Squad.IsOnPathToSquadObjective(RouteGoal)
	|| (!bAvoidanceMappingUpdateVehicles && Vehicle(Pawn) != None)
	|| Skill + Tactics < 3.0)
		return;

	//Refresh route if needed (ForceRefreshRouteInterval is set from AvoidanceMapper, so this should only run if it's active)
	if (WorldInfo.TimeSeconds - LastForceRefreshRouteTime > ForceRefreshRouteInterval
	&& (bAvoidanceMappingUpdateVehicles || Vehicle(Pawn) == None)) {
		bForceRefreshRoute = true;
		LastForceRefreshRouteTime = WorldInfo.TimeSeconds;
	}

	//Avoid waiting for lifts that are already moving and above us (will have to wait for them)
	foreach AvoidanceMapper.LiftExits(LE) {
		//LE.MyLiftCenter != None && LE.MyLiftCenter.MyLift != None checked when building LE cache
		if (LE.MyLiftCenter.MyLift.Location.Z > LE.Location.Z
		&& LE.MyLiftCenter.MyLift.Location.Z > Pawn.Location.Z + Pawn.MaxStepHeight) {
			LE.MyLiftCenter.TransientCost += ADDCOSTLIFTWAIT;
			LE.TransientCost += ADDCOSTLIFTWAIT;
		}
	}

	//Iterate our RouteAvoidanceCache and add the costs -- TODO Partition by team? See changeset fea8cbb6922f99941086496409298cb18f5c4940
	MapperIndex = UTPlayerReplicationInfo(PlayerReplicationInfo).OrdersIndex;
	for (i = 0; i < AvoidanceMapper.RouteAvoidanceCache.Length; i++) {
		C = AvoidanceMapper.RouteAvoidanceCache[i].OwningController;
		if (i == MapperIndex || C == None || C.Pawn == None
		|| (!bAvoidanceMappingAvoidTeammates && WorldInfo.GRI.OnSameTeam(Self, C)))
			continue;

		//Calculate their relative strength - if it's below -0.99 ignore 'em
		RelStr = (RelativeStrength(C.Pawn) + 1.0);
		if (RelStr < 0.01)
			continue;
		RelStr *= ADDCOSTENEMY / ((bAvoidanceMappingUsePlayerCount)
			? float(WorldInfo.Game.NumPlayers + WorldInfo.Game.NumBots)
			: 8.0);
		foreach AvoidanceMapper.RouteAvoidanceCache[i].Points(N) {
			if (N != None) {
				N.TransientCost += RelStr;
				//`Log("foxLog: Setting avoidance on" @ N @ "with cost" @ N.TransientCost @ "for idx" @ MapperIndex @ "vs" @ i);
			}
		}
	}
}

//fox: If we've been standing in an area for too long and really shouldn't be, we might be stuck on a bad waypoint etc. - try to get out of it
function bool CheckIfStuck()
{
	local UTPawn P;
	local UTVehicle V;
	local bool bReallyStuck;
	local float VelocityZ;

	//If we don't exist, we're supposed to be still, or we're at and can trace our RouteGoal, count ourselves as unstuck
	if (Pawn != None && !Pawn.bStationary
	&& (DoorTarget != None || Focus == None || !Pawn.IsFiring() || !CanAttack(Focus))
	&& (PendingMover == None || IsZero(PendingMover.Velocity))
	&& (DefensePoint == None || !Pawn.ReachedDestination(DefensePoint.GetMoveTarget()))
	&& (MoveTarget == None || !Pawn.ReachedDestination(MoveTarget) || !FastTrace(MoveTarget.Location, Pawn.Location))) {
		//Update positions every CheckIfStuckInterval seconds
		if (WorldInfo.TimeSeconds - LastCheckIfStuckTime > CheckIfStuckInterval) {
			LastCheckIfStuckTime = WorldInfo.TimeSeconds;
			OldLocation[2] = OldLocation[1];
			OldLocation[1] = OldLocation[0];
			OldLocation[0] = Pawn.Location;
		}

		//Are we stuck at all?
		if (MoveTarget == None || Pawn.FindAnchorFailedTime == WorldInfo.TimeSeconds
		|| VSize(OldLocation[0] - OldLocation[1]) < CheckIfStuckRadius) {
			bReallyStuck = VSize(OldLocation[0] - OldLocation[2]) < CheckIfStuckRadius;

			//Are we stuck on a Hoverboard? Just bail
			if (UTVehicle_Hoverboard(Pawn) != None) {
				LeaveVehicle(true);
				return true;
			}

			//Are we stuck on foot? Possibly dodge; if we're a hero crouch and failing that just suicide
			P = UTPawn(Pawn);
			if (P != None) {
				if (P.IsHero())
					P.bWantsToCrouch = true;
				if (bReallyStuck && WorldInfo.TimeSeconds - LastUnstuckTime > CheckIfStuckInterval * 6.0) {
					P.Suicide();
					return true;
				}
				UnstuckTarget = FindUnstuckDest();
				if (bReallyStuck && UnstuckTarget != None) {
					if (P.Physics == PHYS_Walking && UnstuckTarget.Location.Z - P.Location.Z < P.MaxStepHeight) {
						P.Velocity = Normal(UnstuckTarget.Location - P.Location) * P.GroundSpeed;
						P.Velocity.Z = 0;
						P.Acceleration = vect(0,0,0);
						bNotifyApex = true;
						bPendingDoubleJump = true;
						P.DoJump(false);
					}
					else if (UnstuckNudgeVelocity != -1 && CloseEnoughToGround()
					&& WorldInfo.TimeSeconds - LastUnstuckTime > CheckIfStuckInterval * 4.0) {
						VelocityZ = P.Velocity.Z;
						P.Velocity = 2 * UnstuckNudgeVelocity * VRand();
						P.Velocity.Z += VelocityZ;
					}
					else {
						VelocityZ = UnstuckTarget.Location.Z - P.Location.Z;
						if (bAllowedToImpactJump && P.JumpZ + ImpactJumpZ + MultiJumpZ > VelocityZ)
							SpecialJumpTo(UnstuckTarget, VelocityZ);
					}
					MoveTimer = 1.0;
				}
			}

			//Are we stuck in a vehicle and driving? Try to get unstuck and failing that just bail
			V = UTVehicle(Pawn);
			if (V != None && V.Controller == Self && !V.bIsOnTrack
			&& (UTVehicle_Deployable(V) == None || !UTVehicle_Deployable(V).IsDeployed())
			&& Abs(V.Mesh.BodyInstance.GetUnrealWorldAngularVelocity().Z) < 0.05) {
				if (Squad.SquadObjective != None
				&& WorldInfo.TimeSeconds - LastUnstuckTime > CheckIfStuckInterval * 4.0) {
					if (WorldInfo.TimeSeconds - LastUnstuckTime > CheckIfStuckInterval * 6.0)
						V.VehicleLostTime = WorldInfo.TimeSeconds + 10.0;
					LeaveVehicle(true);
					return true;
				}
				UnstuckTarget = FindUnstuckDest();
				if (UnstuckNudgeVelocity != -1)
					SetTimer(0.01, false, 'NudgeVehicle'); //Also wiggle about a little - unsafe to do now, so call later
				V.SetInputs(0.0, 0.0, 1.0); //Jump!
			}

			//If we have an UnstuckTarget move now
			if (UnstuckTarget != None) {
				GoalString = "Getting Unstuck to" @ UnstuckTarget @ "for" @ WorldInfo.TimeSeconds - LastUnstuckTime;
				if (SetRouteToGoal(UnstuckTarget))
					return true;
				MoveTarget = UnstuckTarget;
				if (StartMoveToward(UnstuckTarget))
					return true;
			}

			//Stop here so we can check if we're still stuck next time
			return false;
		}
	}

	//Are we not stuck? Resume our normal planning
	LastUnstuckTime = WorldInfo.TimeSeconds;

	//Make sure we're not still moving to our UnstuckTarget/RouteGoal directly if not stuck
	if (MoveTarget == UnstuckTarget)
		FindBestPathToward(RouteGoal,false,true);

	UnstuckTarget = None;
	return false;
}

//fox: Called from CheckIfStuck to nudge the vehicle at a safe time
function NudgeVehicle()
{
	local UTVehicle V;
	local bool bReverse;

	//Make sure we're still valid after the time gap
	V = UTVehicle(Pawn);
	if (V != None && V.Mesh != None) {
		bReverse = FRand() < ((V.Throttle < 0) ? 0.8 : 0.2);
		V.Mesh.AddImpulse(UnstuckNudgeVelocity * ((bReverse) ? vect(-1,0,0) : vect(1,0,0)) >> V.Rotation, , , true);
	}
}

//fox: Attempt to actually get unstuck (find a random point to move to, etc.)
function Actor FindUnstuckDest()
{
	local Controller C;

	//If we don't want to use UnstuckTarget routing then bail here
	if (UnstuckTargetInterval == -1)
		return None;

	//Mark our last MoveTarget as a potentially unreachable node since it wasn't working out and find a destination to get unstuck to
	if (NavigationPoint(MoveTarget) != None)
		AddUnreachableNode(NavigationPoint(MoveTarget));

	//Is our last UnstuckTarget still valid? Just use that
	if (UnstuckTarget != None && WorldInfo.TimeSeconds - UnstuckTargetTime < UnstuckTargetInterval
	&& VSize(UnstuckTarget.Location - Pawn.Location) > UnstuckTargetMinRadius)
		return UnstuckTarget;

	UnstuckTargetTime = WorldInfo.TimeSeconds;

	//Try our RouteGoal (really) unless we tried it last time
	if (TryUnstuckDest(RouteGoal))
		return RouteGoal;

	//Try our current anchor unless we tried it last time
	if (TryUnstuckDest(Pawn.Anchor))
		return Pawn.Anchor;

	//Try our last anchor unless we tried it last time
	if (TryUnstuckDest(Pawn.LastAnchor))
		return Pawn.LastAnchor;

	//If another bot/player is available, try to go to them
	foreach WorldInfo.AllControllers(class'Controller', C)
		if (FRand() < 0.4 && TryUnstuckDest(C.Pawn, UnstuckTargetMinRadius + Pawn.GetCollisionRadius()))
			return C.Pawn;

	//All else fails, just use a random dest
	return FindRandomDest();
}
function bool TryUnstuckDest(Actor A, optional float MinRadius = UnstuckTargetMinRadius)
{
	local NavigationPoint N;

	N = NavigationPoint(A);
	if (N != None && UnreachableNodes.Find(N) != -1)
		return false;
	return A != None && A != UnstuckTarget && VSize(A.Location - Pawn.Location) > MinRadius;
}

//fox: Add / Remove an unreachable node to the UnreachableNode array (almost like a queue)
function AddUnreachableNode(NavigationPoint N)
{
	//Note: A single node could in theory be added multiple times; this simply means it'll get additional accumulating cost in the pathing pass
	//`Log("foxLog: Adding unreachable node" @ N $ ", queue size" @ UnreachableNodes.Length);
	UnreachableNodes.AddItem(N);
	if (!IsTimerActive('RemoveUnreachableNode'))
		SetTimer(10.0, false, 'RemoveUnreachableNode');
	bForceRefreshRoute = true;
}
function RemoveUnreachableNode()
{
	if (UnreachableNodes.Length > 0) {
		//`Log("foxLog: Removing unreachable node" @ UnreachableNodes[0] $ ", queue size" @ UnreachableNodes.Length);
		UnreachableNodes.Remove(0, 1);
		SetTimer(10.0, false, 'RemoveUnreachableNode');
	}
}

//fox: If can't find a path, mark as such until next restart
function bool StartMoveToward(Actor O)
{
	if (MoveTarget == None)
		bPathingFailed = true;
	return Super.StartMoveToward(O);
}

//fox: Deal with vehicle pushers since we're allowing extended vehicle pathing -- TODO I wonder if this only happens when driver pawn collides - not vehicle physics mesh
event NotifyPhysicsVolumeChange(PhysicsVolume NewVolume)
{
	local ForcedDirVolume DirVolume;
	local Vector VolDir, MoveDir;

	DirVolume = ForcedDirVolume(NewVolume);
	if (DirVolume != None && MoveTarget != None && Vehicle(Pawn) != None) {
		VolDir = Normal(DirVolume.ArrowDirection);
		MoveDir = Normal(MoveTarget.Location - Pawn.Location);
		//`Log("foxLog:" @ Self @ GetHumanReadableName() @ "smacked into ForcedDirVolume" @ DirVolume @ "with rotation" @ VolDir @ "vs" @ MoveDir @ "(" $ VolDir dot MoveDir $ ")");
		//if (NavigationPoint(MoveTarget) != None)
		//	AddUnreachableNode(NavigationPoint(MoveTarget));
		if (VolDir dot MoveDir < 0)
			LeaveVehicle(false);
	}

	Super.NotifyPhysicsVolumeChange(NewVolume);
}

//fox: Fix issue where bots may get stuck pathing back and forth between objective and super item if needing to backtrack
function bool FindSuperPickup(float MaxDist)
{
	return Super.FindSuperPickup((PlayerReplicationInfo.bHasFlag)
		? 1024.0
		: (RouteGoal != None && !Squad.bFreelance && Squad.SquadObjective != None)
			? VSize(RouteGoal.Location - Pawn.Location) + 3072.0
			: float(class'NavigationPoint'.const.INFINITE_PATH_COST));
}

//fox: Don't use Translocator when actively firing another weapon (unless we're falling dangerously)
function bool CanUseTranslocator()
{
	return (UTWeap_Translocator(Pawn.Weapon) != None || !Pawn.IsFiring()
			|| (Pawn.Physics == PHYS_Falling && (Pawn.Velocity.Z < -Pawn.MaxFallSpeed || !CloseEnoughToGround())))
		&& Super.CanUseTranslocator();
}

//fox: Keep moving when translocating if (relatively) safe to do so
event TranslocateTo(Actor DestinationActor)
{
	local UTWeap_Translocator TW;

	if (DestinationActor == None)
		return;
	TW = UTWeap_Translocator(Pawn.Weapon);
	bPreparingMove = TW == None
		|| TW.TransDisc == None
		|| TW.TransDisc.bDeleteMe;
	TranslocationTarget = DestinationActor;
	ImpactTarget = DestinationActor;
	Focus = DestinationActor;
	if (TW != None) {
		Pawn.InvManager.PendingWeapon = None;
		if (bPreparingMove && CanAttack(DestinationActor)) {
			TW.FireHack();
			if (TW.TransDisc != None) {
				TW.TransDisc.SetCollision(false, false);
				bPreparingMove = false;
			}
		}
	}
	else
		SwitchToBestWeapon();
}

//fox: Check if we're dealing with a DoorMarker - and make sure our best DoorTarget weapon is out if so
function CheckDoorTarget()
{
	DoorTarget = DoorMarker(Focus);
	if (DoorTarget == None && InterpActor(Focus) != None)
		DoorTarget = DoorMarker(InterpActor(Focus).MyMarker);
	if (DoorTarget != None && DoorTarget.DoorType == DOOR_Shoot)
		SwitchToBestWeapon();
}

//fox: Extra weapon switching behaviors and weights
exec function SwitchToBestWeapon(optional bool bForceNewWeapon)
{
	local class<Inventory> T;
	local Weapon Weap;
	local UTPawn UTEnemy;
	local UTWeap_Translocator TW;
	local float DoorRating;

	if (Pawn == None || Pawn.InvManager == None)
		return;

	//Don't switch weapons too often
	if (LastWeaponSwitchTime > WorldInfo.TimeSeconds)
		return;

	//Don't switch weapons if we're waiting on a Translocator disc
	TW = UTWeap_Translocator(Pawn.Weapon);
	if (TW != None && TW.TransDisc != None && !TW.TransDisc.bDeleteMe && TW.TransDisc.Physics == PHYS_Falling)
		return;

	//Use a preferred list of weapons to shoot shootable doors
	DoorRating = 80.0;
	foreach PreferredDoorWeapons(T) {
		Weap = Weapon(Pawn.FindInventoryType(T));
		if (Weap != None)
			Weap.AIRating = (DoorTarget != None) ? DoorRating : Weap.default.AIRating;
		DoorRating -= 0.1;
	}

	//Use impact hammers on invulnerable enemies - bit of a hack as the Hammer does not use AIRating when an enemy is present
	Weap = Weapon(Pawn.FindInventoryType(class'UTWeap_ImpactHammer'));
	if (Weap != None) {
		//Also try it on shielded enemies, but only roll the decision every 20 seconds
		if (WorldInfo.TimeSeconds - DisruptShieldBeltsCheckTime > 20.0) {
			bDisruptShieldBelts = FRand() < 0.3;
			DisruptShieldBeltsCheckTime = WorldInfo.TimeSeconds;
		}
		UTEnemy = UTPawn(Enemy);
		if (Skill + Tactics > 3.0 && Pawn.InvManager != None
		&& UTEnemy != None && UTEnemy.Controller != None && !UTEnemy.IsHero()
		&& (UTEnemy.bIsInvulnerable || UTEnemy.ShieldBeltArmor > ((bDisruptShieldBelts) ? 80 : 150))
		&& VSize(UTEnemy.Location - Pawn.Location) < ((UTEnemy.Controller.CanSee(Pawn)) ? MELEERANGE / 2.0 : MELEERANGE)
		&& LineOfSightTo(UTEnemy)) {
			if (Weap != Pawn.Weapon) {
				StopFiring();
				Pawn.InvManager.ServerSetCurrentWeapon(Weap);

				//Don't switch weapons too often
				LastWeaponSwitchTime = WorldInfo.TimeSeconds + Pawn.Weapon.PutDownTime;
			}
			return;
		}
	}

	Super.SwitchToBestWeapon(bForceNewWeapon);

	//If we're not actually switching weapons, stop here
	if (Pawn.InvManager.PendingWeapon == None || Pawn.InvManager.PendingWeapon == Pawn.Weapon)
		return;

	//Otherwise, stop firing immediately
	//Note: Charging / charged weapons will continue holding if PendingWeapon.GetAIRating() < 10.0
	StopFiring();

	//If we're switching away from the Translocator, leave a disc somewhere
	if (TW != None && UTWeap_Translocator(Pawn.InvManager.PendingWeapon) == None) {
		//First try to translocate to (or retrieve) our old one if it exists
		SwitchToBestWeaponTranslocateHack();
		TranslocationTarget = (Pawn.Anchor != None) ? Pawn.Anchor : MoveTarget; //MoveTarget might be none, doesn't matter
		TW.SetTimer(0.01, false, 'ProjectileFire');
	}

	//If we're switching to the Translocator, and we have a disc somewhere, translocate to it
	TW = UTWeap_Translocator(Pawn.InvManager.PendingWeapon);
	if (TW != None && UTWeap_Translocator(Pawn.Weapon) == None)
		SetTimer(Pawn.Weapon.PutDownTime + TW.EquipTime, false, 'SwitchToBestWeaponTranslocateHack');

	//Don't switch weapons too often
	LastWeaponSwitchTime = WorldInfo.TimeSeconds + Pawn.Weapon.PutDownTime;
}
function SwitchToBestWeaponTranslocateHack()
{
	local UTWeap_Translocator TW;

	if (Pawn == None)
		return;

	//Try to find our Translocator, we either have it out or are switching to it - if neither, bail here
	TW = UTWeap_Translocator(Pawn.Weapon);
	//if (TW == None)
	//	TW = UTWeap_Translocator(Pawn.InvManager.PendingWeapon); //TODO I don't think we'll ever need this
	if (TW == None)
		return;

	//We're either recalling our disc or warping to it, so if there's no need to do so just bail here
	if (TW.TransDisc == None || TW.TransDisc.bDeleteMe)
		return;
	if (RouteGoal != None
	&& VSize(RouteGoal.Location - TW.TransDisc.Location) < VSize(RouteGoal.Location - Pawn.Location)
	&& CloseEnoughToGround(TW.TransDisc))
		TW.CustomFire();
	else
		TW.FireHack();
}

//fox: Always set LastFireTarget and Focus when going to FireWeaponAt
function bool FireWeaponAt(Actor A)
{
	if (UTVehicle(Pawn) == None || !UTVehicle(Pawn).bShouldLeaveForCombat) {
		LastFireTarget = A;
		Focus = A;
	}

	//Make sure we check DoorTarget first
	CheckDoorTarget();

	//Also DoorTarget hack to trick DoorMarker code to let us move
	return Super.FireWeaponAt(A) || DoorTarget != None;
}

//fox: Hack to always MarkShootSpotsFor NodeObjectives as their TellBotHowToDisable code is bugged
function DoRangedAttackOn(Actor A)
{
	if (UTOnslaughtNodeObjective(A) != None && Pawn != None)
		UTOnslaughtNodeObjective(A).MarkShootSpotsFor(Pawn);
	Super.DoRangedAttackOn(A);
}

//fox: Force a UTBot StopFiring (so no charge-weapon checks etc.) when we've died, so we release charged weapons like players
function PawnDied(Pawn P)
{
	Super(UTBot).StopFiring();
	Super.PawnDied(P);

	//Also if we were the SquadLeader, PickNewLeader so the offensive / whatever doesn't completely stall
	if (Squad.SquadLeader == Self)
		Squad.PickNewLeader();
}

//fox: More reliable firing and weapon charging
function StopFiring()
{
	//If we're not dumb and can't trace our target, hold charging weapons as long as possible
	//This is very dependent on the LastFireTarget set in FireWeaponAt
	if (Skill + ReactionTime > 3.0 && Pawn != None && Pawn.Health > 0 && Pawn.InvManager != None
	&& Pawn.Weapon != None && Pawn.IsFiring() && Pawn.FireOnRelease()
	&& (Focus == None || Focus != LastFireTarget || Pawn.NeedToTurn(FocalPoint) || !CanAttack(Focus))
	&& (Pawn.InvManager.PendingWeapon == None || Pawn.InvManager.PendingWeapon.GetAIRating() < 10.0)) {
		bCanFire = true;
		bStoppedFiring = false;
		return;
	}

	Super.StopFiring();
}
function bool ShouldFireAgain()
{
	//Charging weapon handling
	//Note: StopFiring's charge weapon check ensures charged weapons still won't fire without a valid target
	if (UTWeapon(Pawn.Weapon) != None) {
		//HACK ShouldFireAgain has a small bug for FireOnRelease weapons that will continue firing if FullyCharged
		if (UTWeapon(Pawn.Weapon).IsFullyCharged())
			return false;

		//HACK If we're a charging weapon, try to randomly hold some charge (depending on skill) to throw off our opponent
		//Especially important for BioRifle as we force ShouldFireOnRelease and there's no real AI for that
		if (Pawn.IsFiring() && Pawn.FireOnRelease() && FRand() < 0.1 * FMin(Skill, 5.0))
			return true;
	}

	//Hold the trigger on the Stinger longer due to wind-up (unfortunately there's no windup weapon flag)
	if (UTWeap_Stinger(Pawn.Weapon) != None && Pawn.Weapon.CurrentFireMode == 0
	&& (WorldInfo.TimeSeconds - LastSeenTime < 2.0 || FRand() < 0.5))
		return true;

	//Use skill-based LastCanAttackCheckTime comparisons instead of flat 1.0 to stop blowing ourselves up
	if (WorldInfo.TimeSeconds - LastCanAttackCheckTime > 1.0 / FMax(1.0, Skill + ReactionTime)) {
		LastCanAttackCheckTime = WorldInfo.TimeSeconds;
		if (!CanAttack(Focus))
			return false;
	}

	//If we have a pending weapon, don't keep melee weapon charged if we're no longer engaged (so we can switch)
	if (Pawn.InvManager.PendingWeapon != None && Pawn.InvManager.PendingWeapon != Pawn.Weapon
	&& Pawn.Weapon != None && Pawn.Weapon.bMeleeWeapon
	&& (Focus == None || Focus != LastFireTarget || VSize(Focus.Location - Pawn.Location) > MELEERANGE))
		return false;

	return Super.ShouldFireAgain();
}

//fox: Don't snipe when under fire or if we wouldn't be able to see when crouched
function bool IsSniping()
{
	local Rotator TurnRot;

	TurnRot.Yaw = Pawn.Rotation.Yaw;
	return WorldInfo.TimeSeconds - LastUnderFire > 2.0
		&& FastTrace(Pawn.Location + (vect(512,0,-16) >> TurnRot), Pawn.Location + vect(0,0,-16))
		&& Super.IsSniping();
}

//fox: Unset WarningProjectile if its speed is zero (fixes divide-by-zero spam)
function ReceiveProjectileWarning(Projectile Proj)
{
	if (WarningProjectile != None && WarningProjectile.Speed == 0)
		WarningProjectile = None;
	Super.ReceiveProjectileWarning(Proj);
}

//fox: Allow crouching with more than just sniping weapons
function CheckIfShouldCrouch(vector StartPosition, vector TargetPosition, float Probability)
{
	local vector HitNormal, HitLocation, ProjStart;

	if (WorldInfo.bUseConsoleInput || !Pawn.bCanCrouch
	|| (!Pawn.bIsCrouched && FRand() > Probability)
	|| Skill < 3.0 * FRand()) {
		Pawn.bWantsToCrouch = false;
		return;
	}

	//If we're not crouched and can hit our target, don't adjust unless we randomly decide to
	ProjStart = Pawn.GetWeaponStartTraceLocation() + StartPosition - Pawn.Location;
	if (!Pawn.bIsCrouched && FRand() < 0.1 * Skill + ((UTWeapon(Pawn.Weapon) != None && UTWeapon(Pawn.Weapon).bSniping) ? 0.05 : 0.25)
	&& Trace(HitLocation, HitNormal, TargetPosition, ProjStart, false) != None)
		return;

	ProjStart.Z += ((Pawn.bIsCrouched) ? 1.0 : -1.0) * (Pawn.CylinderComponent.default.CollisionHeight - Pawn.CrouchHeight);
	if (Trace(HitLocation, HitNormal, TargetPosition, ProjStart, false) != None)
		Pawn.bWantsToCrouch = !Pawn.bWantsToCrouch;
}

//fox: Allow dodging more often, particularly with short distances and walls (does NOT call super function)
event MayDodgeToMoveTarget()
{
	local vector Dir, X, Y, Z, DodgeV, TraceStart, TraceEnd, HitLocation, HitNormal;
	local rotator TurnRot;
	local Actor HitActor;
	local float RealJumpZ;
	local float RealGroundSpeed;

	//Enforce DodgeResetTimestamp and MultiDodgeRemaining like we do with players
	if (MoveTarget == None || UTPawn(Pawn) == None || (Pawn.Physics != PHYS_Walking && Pawn.Physics != PHYS_Falling)
	|| UTPawn(Pawn).DodgeResetTimestamp > WorldInfo.TimeSeconds || UTPawn(Pawn).MultiDodgeRemaining <= 0
	|| Skill + Jumpiness < 4.5 + 1.5 * FRand())
		return;

	Dir = MoveTarget.GetDestination(Self) - Pawn.Location;
	Destination = MoveTarget.Location;
	TurnRot.Yaw = Pawn.Rotation.Yaw;
	GetAxes(TurnRot, X, Y, Z);

	if (Abs(X Dot Dir) > Abs(Y Dot Dir)) {
		if (X Dot Dir > 0)
			UTPawn(Pawn).CurrentDir = DCLICK_Forward;
		else
			UTPawn(Pawn).CurrentDir = DCLICK_Back;
	}
	else if (Y Dot Dir < 0)
		UTPawn(Pawn).CurrentDir = DCLICK_Left;
	else
		UTPawn(Pawn).CurrentDir = DCLICK_Right;

	//Borrowed from UTPawn to allow more wall-dodging
	if (Pawn.Physics == PHYS_Falling) {
		if (UTPawn(Pawn).CurrentDir == DCLICK_Forward)
			TraceEnd = -X;
		else if (UTPawn(Pawn).CurrentDir == DCLICK_Back)
			TraceEnd = X;
		else if (UTPawn(Pawn).CurrentDir == DCLICK_Left)
			TraceEnd = Y;
		else if (UTPawn(Pawn).CurrentDir == DCLICK_Right)
			TraceEnd = -Y;
		TraceStart = Location - (Pawn.CylinderComponent.CollisionHeight - 16) * vect(0,0,1) + TraceEnd * (Pawn.CylinderComponent.CollisionRadius - 16);
		TraceEnd = TraceStart + TraceEnd * 40.0;
		HitActor = Trace(HitLocation, HitNormal, TraceEnd, TraceStart, false, vect(16,16,16));

		if (HitActor == None || HitNormal.Z < -0.1)
			return;
		if (!HitActor.bWorldGeometry) {
			if (!HitActor.bBlockActors)
				return;
			if (Pawn(HitActor) != None && Vehicle(HitActor) == None)
				return;
		}
	}

	bPlannedJump = true;
	bInDodgeMove = true; //For setting DodgeResetTimestamp
	UTPawn(Pawn).PerformDodge(UTPawn(Pawn).CurrentDir, Normal(Dir), vect(0,0,0));

	// if below, make sure really far
	if (Dir.Z < -1 * Pawn.GetCollisionHeight()) {
		Pawn.Velocity.Z = 0.0;
		RealJumpZ = Pawn.JumpZ;
		DodgeV = UTPawn(Pawn).BotDodge(vect(1,0,0));
		Pawn.JumpZ = DodgeV.Z;
		DodgeV.Z = 0.0;
		RealGroundSpeed = Pawn.GroundSpeed;
		Pawn.GroundSpeed = VSize(DodgeV);
		Pawn.SuggestJumpVelocity(Pawn.Velocity, Destination, Pawn.Location);
		Pawn.GroundSpeed = RealGroundSpeed;
		Pawn.JumpZ = RealJumpZ;
	}
	Pawn.Acceleration = vect(0,0,0);
}

//fox: Always NotifyFallingHitWall (which can TryWallDodge) when falling
event bool NotifyHitWall(vector HitNormal, actor Wall)
{
	//Also counteract the weird location change when swimming
	if (Pawn.Physics == PHYS_Swimming) {
		//`Log("foxLog:" @ Self @ "swimming NotifyHitWall" @ HitNormal @ Wall @ Pawn.HeadVolume);
		if (MoveTarget != None && (Pawn.HeadVolume == None || !Pawn.HeadVolume.bWaterVolume)) {
			ImpactVelocity = vect(0,0,0);
			Pawn.JumpOutOfWater(Normal(Movetarget.Location - Pawn.Location));
			bNotifyApex = true;
			bPendingDoubleJump = true;
		}
		else if (bSwimmingHitWallFix && IsZero(SwimmingFixOldLocation)) {
			SwimmingFixOldLocation = Pawn.Location;
			SetTimer(0.001, false, 'TimedSwimmingFixResetLocation');
		}
	}

	if (Pawn.Physics == PHYS_Falling)
		NotifyFallingHitWall(HitNormal, Wall);
	return Super.NotifyHitWall(HitNormal, Wall);
}
function TimedSwimmingFixResetLocation()
{
	if (Pawn != None)
		Pawn.SetLocation(SwimmingFixOldLocation);
	SwimmingFixOldLocation = vect(0,0,0);
}

//fox: Also try a MayDodgeToMoveTarget
event NotifyFallingHitWall(vector HitNormal, actor HitActor)
{
	MayDodgeToMoveTarget();
	Super.NotifyFallingHitWall(HitNormal, HitActor);
}

//fox: Enforce DodgeResetTimestamp and MultiDodgeRemaining like we do with players
function bool TryWallDodge(vector HitNormal, actor HitActor)
{
	if (UTPawn(Pawn) == None || (Pawn.Physics != PHYS_Walking && Pawn.Physics != PHYS_Falling)
	|| UTPawn(Pawn).DodgeResetTimestamp > WorldInfo.TimeSeconds || UTPawn(Pawn).MultiDodgeRemaining <= 0)
		return false;
	if (Super.TryWallDodge(HitNormal, HitActor)) {
		bInDodgeMove = true; //For setting DodgeResetTimestamp
		return true;
	}
	return false;
}

//fox: Set DodgeResetTimestamp when we land -- TODO && bInDodgeMove, but this fixes some edge cases (TODO ...like what?)
event bool NotifyLanded(vector HitNormal, Actor FloorActor)
{
	if (UTPawn(Pawn) != None)
		UTPawn(Pawn).DodgeResetTimestamp = WorldInfo.TimeSeconds + UTPawn(Pawn).DodgeResetTime;
	return Super.NotifyLanded(HitNormal, FloorActor);
}

//fox: Return a healing weapon that can heal A
function UTWeapon GetHealingWeaponFor(Actor A)
{
	local Pawn CheckPawn;
	local UTWeapon W;

	if (Vehicle(Pawn) != None)
		CheckPawn = Vehicle(Pawn).Driver;
	else
		CheckPawn = Pawn;

	if (CheckPawn == None || CheckPawn.InvManager == None)
		return None;

	W = UTWeapon(CheckPawn.Weapon);
	if (W != None && W.CanHeal(A) && W.HasAnyAmmo()
	&& VSize(A.Location - Pawn.Location) < W.WeaponRange)
		return W;
	foreach CheckPawn.InvManager.InventoryActors(class'UTWeapon', W)
		if (W.CanHeal(A) && W.HasAnyAmmo()
		&& VSize(A.Location - Pawn.Location) < W.WeaponRange)
			return W;

	return None;
}

//fox: Are we capable of healing a vehicle?
function bool CanHealVehicle(UTVehicle V)
{
	local UTWeapon HealWeap;

	if (V == None || V.Health >= V.HealthMax
	|| WorldInfo.TimeSeconds == LastUnderFire
	|| PriorityObjective())
		return false;

	if (UTVehicle(Pawn) != None && !UTVehicle(Pawn).bShouldLeaveForCombat
	&& Enemy != None && LineOfSightTo(Enemy))
		return false;

	HealWeap = GetHealingWeaponFor(V);
	if (HealWeap == None)
		return false;

	//If the vehicle we're trying to heal is our vehicle, we're good at this point
	if (UTVehicle(Pawn) == V)
		return true;

	//If this is deathmatch, don't bother as somebody will probably just steal it
	//TODO possibly allow this, though bots get confused with link fire mode since vehicle is never set to their team
	if (Squad.Team == None || Squad.Team.TeamIndex == 255)
		return false;

	//If this vehicle is teamlocked/occupied and not on the same team, bail
	if ((V.bTeamLocked || V.Occupied()) && !V.TeamLink(Squad.Team.TeamIndex))
		return false;

	return FastTrace(V.Location, Pawn.Location);
}

//fox: Attempt to heal a vehicle
function bool TryToHealVehicle(UTVehicle V)
{
	if (V != None && CanHealVehicle(V)) {
		GoalString = "TryToHealVehicle" @ V;

		//Are we in a vehicle? Well CanHealVehicle passed, so let's get out
		if (Vehicle(Pawn) != None) {
			LeaveVehicle(true);
			return true;
		}

		SetRouteToGoal(V);

		//If our weapon is out and can heal, go for it
		if (UTWeapon(Pawn.Weapon) != None && UTWeapon(Pawn.Weapon).CanHeal(V)) {
			LastCanAttackCheckTime = WorldInfo.TimeSeconds;
			//Hack to get bots to use alt-fire on enemy unlocked vehicles
			if (!WorldInfo.GRI.OnSameTeam(Pawn, V))
				Enemy = V;
			DoRangedAttackOn(V);
			//Unset Enemy, otherwise we'll keep this as our enemy when entering it leading to hilarious but counterproductive results
			//This might possibly not be reached due to RangedAttack state, but TimedUpdateEnemy will handle it if so
			if (Enemy == V)
				Enemy = None;
		}
		//Otherwise switch to healing weapon
		else {
			SwitchToBestWeapon(); //Should rate itself highly as our RouteGoal is V

			//If for some reason our PendingWeapon is not a healing weapon like it should be, bail here
			if (UTWeapon(Pawn.InvManager.PendingWeapon) == None || !UTWeapon(Pawn.InvManager.PendingWeapon).CanHeal(V))
				return false;
		}

		//CanHealVehicle passed so we're good even if we're waiting on a weapon
		return true;
	}

	return false;
}

//fox: Set VehicleEnterTime to ensure we don't bail too quickly, and always try to heal vehicles
function EnterVehicle(Vehicle V)
{
	VehicleEnterTime = WorldInfo.TimeSeconds;

	//If we're in a hoverboard, get out of that first before getting in this
	if (UTVehicle_Hoverboard(Pawn) != None)
		LeaveVehicle(false);

	//Always try to heal vehicle first
	if (UTVehicle(V) != None && TryToHealVehicle(UTVehicle(V)))
		return;

	//HACK Ensure deployable vehicles are basically stopped before allowing deployment
	if (UTVehicle_Deployable(V) != None)
		UTVehicle_Deployable(V).MaxDeploySpeed = 1.0;

	Super.EnterVehicle(V);
}

//fox: Only leave vehicle if allowed
function LeaveVehicle(bool bBlocking)
{
	//Lessen the AirVehicle PushForce so we stop punting our ride off tall towers
	if (UTAirVehicle(Pawn) != None)
		UTAirVehicle(Pawn).PushForce = FMax(UTAirVehicle(Pawn).default.PushForce - VSize(Pawn.Mesh.BodyInstance.GetUnrealWorldVelocity()), 0.0);

	//Clear any repeating Scorpion / Viper SelfDestruct timers / flags as these may end up kicking a subsequent player driver out
	if (UTVehicle_Scorpion(Pawn) != None) {
		UTVehicle_Scorpion(Pawn).bAISelfDestruct = false;
		Pawn.ClearTimer('CheckScriptedSelfDestruct'); //why was this timer looping in the first place?
	}
	if (UTVehicle_Viper(Pawn) != None)
		UTVehicle_Viper(Pawn).bScriptedSelfDestruct = false;

	//Reset deployable vehicle speed
	if (UTVehicle_Deployable(Pawn) != None)
		UTVehicle_Deployable(Pawn).MaxDeploySpeed = UTVehicle_Deployable(Pawn).default.MaxDeploySpeed;

	if (bBlocking || CanLeaveVehicle())
		Super.LeaveVehicle(bBlocking);
}

//fox: Don't leave a vehicle until MinVehicleTime has passed - called from LeaveVehicle above (non-blocking) or from LeavingVehicle (blocking)
function bool CanLeaveVehicle()
{
	return UTVehicle_Hoverboard(Pawn) != None || WorldInfo.TimeSeconds - VehicleEnterTime > MinVehicleTime;
}

//fox: Are we close enough to the ground?
function bool CloseEnoughToGround(optional Actor A = Pawn)
{
	local Vector HitLocation, HitNormal;
	local Actor T;

	T = Trace(HitLocation, HitNormal, A.Location - vect(0,0,1024), A.Location, true);
	//if (T != None)
	//	`Log("foxLog: CloseEnoughToGround hit Actor" @ T @ "from" @ A);
	return T != None && (PhysicsVolume(T) == None || !PhysicsVolume(T).bPainCausing || PhysicsVolume(T).DamagePerSec <= 0.0);
}

//fox: Is this an idle focus point (something we should not shoot at)
function bool IsIdleFocusPoint(Actor A)
{
	return A == None || (NavigationPoint(A) != None && DoorMarker(A) == None && Objective(A) == None
			&& (UTTranslocatorDest(A) == None
				|| (UTWeap_Translocator(Pawn.Weapon) == None
					&& UTWeap_Translocator(Pawn.InvManager.PendingWeapon) == None)));
}

//fox: Are we in a real vehicle (not just a hoverboard)?
function bool InRealVehicle(optional Controller C = Self)
{
	return Vehicle(C.Pawn) != None && UTVehicle_Hoverboard(C.Pawn) == None;
}

//fox: Check whether node is valid for current pawn (called on vehicle transition, also used by ValidVehicleForBotRoute in SquadAI)
function bool InvalidNodeForVehicle(NavigationPoint N, optional Vehicle V = Vehicle(Pawn),
	optional float ColRadius = -1, optional float ColHeight = -1)
{
	if (V == None && Pawn != None)
		V = Pawn.GetVehicleBase();
	if (V == None && (ColRadius < 0 || ColHeight < 0))
		return false;
	if (ColRadius < 0)
		ColRadius = V.GetCollisionRadius();
	if (ColHeight < 0)
		ColHeight = V.GetCollisionHeight();

	return ColRadius >= N.MaxPathSize.Radius
		|| ColHeight >= N.MaxPathSize.Height
		|| ForcedDirVolume(N.PhysicsVolume) != None
		|| ((V == None || !V.bCanFly) && (LiftCenter(N) != None || UTJumpPad(N) != None));
}

//fox: Increased sight radius options, including decreased sight radius under low visibility
function AdjustSightRadius()
{
	local UTMapInfo MapInfo;

	MapInfo = UTMapInfo(WorldInfo.GetMapInfo());
	if (MapInfo != None && MapInfo.VisibilityModifier < 1.0)
		Pawn.SightRadius = Pawn.default.SightRadius * 0.75; //Use slightly lower than vanilla
	else
		Pawn.SightRadius = (MaxSightRadius != -1) ? MaxSightRadius : Pawn.default.SightRadius;
}
function SetPeripheralVision()
{
	Super.SetPeripheralVision();
	if (Pawn != None)
		AdjustSightRadius();
}
function bool CanAttack(Actor Other)
{
	local bool ret;

	if (Pawn != None)
		AdjustSightRadius();
	ret = Super.CanAttack(Other);

	//Handle deploy hack as we may have started 0.01s ServerToggleDeploy timer
	DeployHack();

	return ret;
}
event float AdjustAimError(float TargetDist, bool bInstantProj)
{
	if (Pawn != None)
		AdjustSightRadius();
	return Super.AdjustAimError(TargetDist, bInstantProj);
}

//fox: If we ordered to a UTHoldSpot, refresh that point; also reset auto-toss times and update vehicle paths if needed
function Restart(bool bVehicleTransition)
{
	local NavigationPoint N;

	//Force a UTBot StopFiring (so no charge-weapon checks etc.) when restarting
	Super(UTBot).StopFiring();

	Super.Restart(bVehicleTransition);

	LastUnstuckTime = WorldInfo.TimeSeconds;
	SwimmingFixOldLocation = vect(0,0,0);

	//Accept more fall damage for a shorter route
	if (Pawn != None)
		Pawn.AIMaxFallSpeedFactor = 2.0;

	//Check invalid nodes (done here as it's largely based on our collision size)
	InvalidNodes.Length = 0;
	if (ExtendedVehiclePathingLevel > 0 && InRealVehicle()) {
		foreach WorldInfo.AllNavigationPoints(class'NavigationPoint', N)
			if (N != None && InvalidNodeForVehicle(N))
				InvalidNodes.AddItem(N);
	}
	bReachedParkingSpot = false;
	bPathingFailed = false;
	bForceRefreshRoute = true;

	//Set whether we're using parking spots
	bUsingParkingSpots = (ExtendedVehiclePathingLevel == -1 && InRealVehicle()) || GetDeployableVehicle() != None;

	//Set bGatherVehiclePassengers here, as we really don't want to suddenly decide to do this after driving off
	bGatherVehiclePassengers = Pawn.bCanFly; //TODO or FRand() < 0.25 for land vehicles, but it's touchy

	//Always SetPeripheralVision, even on bVehicleTransition
	SetPeripheralVision();

	//Increase our MaxStepHeight to deal with some of the sloppy jumping
	if (UTPawn(Pawn) != None)
		UTPawn(Pawn).SetMaxStepHeight(FMax(Pawn.MaxStepHeight, Pawn.default.MaxStepHeight + 32.0));

	//If we're (re)spawning...
	if (!bVehicleTransition) {
		LastWeaponTossTime = WorldInfo.TimeSeconds - WeaponTossInterval + 10.0; //Wait 10 seconds so we don't toss Enforcers everywhere for Weapon Locker spawns
		ForcedFlagDropTime = WorldInfo.TimeSeconds - FlagTossInterval;

		//Reset the DisruptShieldBelts checks
		bDisruptShieldBelts = false;
		DisruptShieldBeltsCheckTime = WorldInfo.TimeSeconds - 20.0;

		//Reset our DefensePoint if it was a HoldSpot (specifically ordered)
		if (UTHoldSpot(CachedDefensePoint) != None)
			DefensePoint = CachedDefensePoint;

		//Register our various timers
		RegisterTimers();

		//Pull any needed mutator data (AvoidanceMapper, etc.)
		//HACK: Put here to fix campaign bugs, should probably be in PostBeginPlay otherwise
		PullMutatorData();
	}
}

//fox: Register misc timers not really covered elsewhere
function RegisterTimers()
{
	//Additional CheckIfStuckTimer, since we may need this in-between nodes
	if (CheckIfStuckInterval > 0 && !IsTimerActive('CheckIfStuck'))
		SetTimer(CheckIfStuckInterval, true, 'CheckIfStuck');

	//Combat helper timers
	if (!IsTimerActive('TimedUpdateEnemy'))
		SetTimer(0.65, true, 'TimedUpdateEnemy');
	if (!IsTimerActive('TimedUpdateFocus'))
		SetTimer(0.45, true, 'TimedUpdateFocus');

	if (!IsTimerActive('TimedTranslocateHack'))
		SetTimer(0.25, true, 'TimedTranslocateHack');
	if (!IsTimerActive('TimedJumpDodgeHack'))
		SetTimer(3.0 * FRand(), false, 'TimedJumpDodgeHack'); //Modified by ExecuteWhatToDoNext, so reset every run
	if (!IsTimerActive('TimedWallDodgeHack'))
		SetTimer(5.0 / FMax(0.1, Skill + Jumpiness), true, 'TimedWallDodgeHack');
	if (!IsTimerActive('TimedCheckHoverboard'))
		SetTimer(2.0 / FMax(0.1, Skill + ReactionTime), true, 'TimedCheckHoverboard');
}
function UnRegisterTimers()
{
	if (IsTimerActive('CheckIfStuck'))
		ClearTimer('CheckIfStuck');

	if (IsTimerActive('TimedUpdateEnemy'))
		ClearTimer('TimedUpdateEnemy');
	if (IsTimerActive('TimedUpdateFocus'))
		ClearTimer('TimedUpdateFocus');

	if (IsTimerActive('TimedTranslocateHack'))
		ClearTimer('TimedTranslocateHack');
	if (IsTimerActive('TimedJumpDodgeHack'))
		ClearTimer('TimedJumpDodgeHack');
	if (IsTimerActive('TimedWallDodgeHack'))
		ClearTimer('TimedWallDodgeHack');
	if (IsTimerActive('TimedCheckHoverboard'))
		ClearTimer('TimedCheckHoverboard');
}

//fox: Potentially find new enemy (called on timer)
function TimedUpdateEnemy()
{
	if (Squad == None || Pawn == None)
		return;

	//Clear our LastFireTarget if it's this enemy (so that we don't stay locked to an old enemy if we switch)
	if (LastFireTarget == Enemy || LastFireTarget == Pawn)
		LastFireTarget = None;

	//If we do have an enemy and they're somehow now on our team lose 'em immediately
	if (Enemy != None && WorldInfo.GRI.OnSameTeam(Pawn, Enemy))
		Enemy = None;

	//Look for new enemy (regardless of whether we have one or not)
	Squad.FindNewEnemyFor(Self, Enemy != None && CanSee(Enemy));
}

//fox: Potentially find something interesting to look at (called on timer)
function TimedUpdateFocus()
{
	local UTGameObjective O, FirstO;
	local UTProj_TransDisc T;

	//Don't track targets if we have Translocator out or we're in a bFollowLookDir vehicle that can't strafe (Hoverboard)
	if (Squad == None || Pawn == None || Pawn.Weapon == None || Pawn.InvManager == None || DoorTarget != None
	|| UTWeap_Translocator(Pawn.Weapon) != None || UTWeap_Translocator(Pawn.InvManager.PendingWeapon) != None
	|| (UTVehicle(Pawn) != None && UTVehicle(Pawn).bFollowLookDir && !UTVehicle(Pawn).bCanStrafe))
		return;

	//Always try our LastFireTarget first
	if (TryForceFireTargetOn(LastFireTarget))
		return;

	//Otherwise try to use a shootable objective
	if (TryForceFireTargetOn(Squad.SquadObjective))
		return;
	if (Squad.Team != None && Squad.Team.AI != None) {
		FirstO = Squad.Team.AI.Objectives;
		for (O = FirstO; O != None; O = (O.NextObjective == FirstO) ? None : O.NextObjective)
			if (TryForceFireTargetOn(O))
				return;
	}

	//Otherwise try to use our enemy
	if (TryForceFireTargetOn(Enemy))
		return;

	//Otherwise try to disrupt a translocator disc
	for (T = UTGame(WorldInfo.Game).BeaconList; T != None; T = T.NextBeacon)
		if (!T.Disrupted() && T.Instigator != None
		&& !WorldInfo.GRI.OnSameTeam(Pawn, T.Instigator)
		&& VSize(T.Location - Pawn.Location) < 1536.0
		&& TryForceFireTargetOn(T))
			return;

	//Otherwise, maybe look around a little at visible nodes if we're looking at a node already
	//Don't do this if we're vehicle-gunning, it just looks bad
	if (Pawn.Controller == Self && IsIdleFocusPoint(Focus) && !Pawn.NeedToTurn(FocalPoint))
		FocusOnRandomVisiblePoint();
}
function bool TryForceFireTargetOn(Actor FireTarget)
{
	local Rotator TurnRot;

	//Always focus on FireTarget, unless it's an invisible pawn or otherwise invalid
	//This allows us to always fire at targets, even in special states such as flying through the air etc.
	//Targets are actually fired upon with TimedFireWeaponAtEnemy, which has been extended to just fire at whatever's focused
	//TODO DeployHack Why do vehicles bug out with GetHealingWeaponFor != None instead of == Pawn.Weapon?
	if (FireTarget != None && !FireTarget.bDeleteMe && !IsIdleFocusPoint(FireTarget)
	&& (!WorldInfo.GRI.OnSameTeam(Pawn, FireTarget) || Pawn.Weapon == GetHealingWeaponFor(FireTarget))
	&& FireTarget != Self && FireTarget != Pawn
	&& (FireTarget == Enemy || (!Pawn.Weapon.bMeleeWeapon && CanAttack(FireTarget)))
	&& (Pawn(FireTarget) == None
		|| (Pawn(FireTarget).Health > 0 && !Pawn(FireTarget).IsInvisible()))
	&& (UTGameObjective(FireTarget) == None
		|| (UTGameObjective(FireTarget).Shootable() && UTGameObjective(FireTarget).IsActive()))
	&& (UTOnslaughtObjective(FireTarget) == None
		|| PlayerReplicationInfo.Team == None
		|| UTOnslaughtObjective(FireTarget).PoweredBy(PlayerReplicationInfo.Team.TeamIndex))
	&& (UTOnslaughtPowernode(FireTarget) == None
		|| UTOnslaughtPowernode(FireTarget).Vulnerability == VS_Vulnerable
		|| WorldInfo.GRI.OnSameTeam(Pawn, FireTarget))
	&& (UTWarfareBarricade(FireTarget) == None
		|| (UTWeapon(Pawn.Weapon) != None && UTWeapon(Pawn.Weapon).bCanDestroyBarricades))) {
		//If FireTarget is our enemy, do some special handling with CanSee
		//This is so we don't focus dead-on to enemies through walls or outside our FOV
		//Also update LastSeenTime etc. since it might not be reliably updated otherwise
		if (FireTarget == Enemy) {
			if (CanSee(Enemy)) {
				LastSeenTime = WorldInfo.TimeSeconds;
				LastSeenPos = Enemy.Location;
				LastSeeingPos = Pawn.Location;
			}
			else {
				if (!FastTrace(LastSeenPos, Pawn.Location)) {
					LastSeenPos = LastSeeingPos;
					LastSeeingPos = Pawn.Location;
					if (!FastTrace(LastSeenPos, Pawn.Location))
						LastSeenPos = LastSeeingPos;
				}
				Focus = None;
				FocalPoint = LastSeenPos;
				if (VSize(LastSeeingPos - Pawn.Location) <= Pawn.GetCollisionRadius()) {
					TurnRot.Yaw = Pawn.Rotation.Yaw;
					LastSeenPos += vect(512,0,0) >> TurnRot;
				}
				return false;
			}
		}

		Focus = FireTarget;
		return true;
	}
	return false;
}
function FocusOnRandomVisiblePoint()
{
	local array<NavigationPoint> VisiblePoints;
	local int i;

	i = UTPlayerReplicationInfo(PlayerReplicationInfo).OrdersIndex;
	if (AvoidanceMapper != None && i < AvoidanceMapper.RouteAvoidanceCache.Length) {
		VisiblePoints = AvoidanceMapper.RouteAvoidanceCache[i].Points;
		if (VisiblePoints.Length > 0)
			Focus = VisiblePoints[Rand(VisiblePoints.Length)];
	}
}

//fox: Dynamically SetCombatTimer based on enemy visibility (slightly slower to react to newly appearing enemies)
function SetCombatTimer()
{
	SetTimer(((Enemy != None && CanSee(Enemy)) ? 1.1 : 1.2) - 0.09 * FMin(10, Skill + ReactionTime), true);
}

//fox: Fire at whatever we have focused
function TimedFireWeaponAtEnemy()
{
	local bool bStingerHackOldHasSlowField;

	if (Pawn == None)
		return;

	TimedUpdateEnemy();
	TimedUpdateFocus();
	SwitchToBestWeapon();

	//HACK Sometimes use secondary fire on Stinger even if close -- TODO we should probably subclass UTWeap_Stinger for this, and use this as fallback if that fails
	if (UTWeap_Stinger(Pawn.Weapon) != None && UTPawn(Focus) != None && FRand() < 0.3) {
		bStingerHackOldHasSlowField = UTPawn(Focus).bHasSlowField;
		UTPawn(Focus).bHasSlowField = true;
	}

	if (Focus != None && !Focus.bDeleteMe && !IsIdleFocusPoint(Focus)
	&& (!WorldInfo.GRI.OnSameTeam(Pawn, Focus) || Pawn.Weapon == GetHealingWeaponFor(Focus)))
		FireWeaponAt(Focus);

	//HACK Undo Stinger secondary fire hack
	if (UTPawn(Focus) != None)
		UTPawn(Focus).bHasSlowField = bStingerHackOldHasSlowField;

	//Sometimes other functions mess with this timer, so enforce it here
	//This also allows dynamic alteration of the timer
	SetCombatTimer();
}

//fox: Handle various Translocator-related hacks
function TimedTranslocateHack()
{
	local UTWeap_Translocator TW;
	local Vector TempVel, TempAcc;
	local bool bFailedTranslocation;

	if (Squad == None || Pawn == None || NextTranslocHackTime > WorldInfo.TimeSeconds)
		return;

	//If we just failed translocation or we're clear to Translocate again and we should be hopping / are under fire / are falling fast...
	bFailedTranslocation = NextTranslocTime > WorldInfo.TimeSeconds + 8.0;
	TW = UTWeap_Translocator(Pawn.Weapon);
	if (TW != None && Squad.AllowTranslocationBy(Self)
	&& (bFailedTranslocation
		|| (NextTranslocTime < WorldInfo.TimeSeconds
			&& (TW.TransDisc == None || TW.TransDisc.bDeleteMe
				|| WorldInfo.TimeSeconds - LastUnderFire < 1.0
				|| (Pawn.Physics == PHYS_Falling
					&& (Pawn.Velocity.Z < -Pawn.MaxFallSpeed || !CloseEnoughToGround())))))) {
		//... immediately translocate if appropriate
		if (TW.TransDisc != None && !TW.TransDisc.bDeleteMe) {
			if (CloseEnoughToGround(TW.TransDisc)) {
				//Also preserve movement past translocation (normally hacked to 0 for bots)
				TempVel = Normal(Pawn.Velocity) * FMin(Pawn.GroundSpeed, VSize(Pawn.Velocity));
				TempVel.Z = FMin(Pawn.JumpZ, Pawn.Velocity.Z);
				TempAcc = Pawn.Acceleration;
				TW.CustomFire();
				//Might have died from CustomFire so recheck Pawn
				if (Pawn != None && !Pawn.bDeleteMe && Pawn.Health > 0) {
					Pawn.Velocity = TempVel;
					Pawn.Acceleration = TempAcc;
				}
			}
		}
		//... or immediately try again
		else if (MoveTarget != None
		&& VSize(MoveTarget.Location - Pawn.Location) > Pawn.GetCollisionRadius())
			TranslocateTo(MoveTarget);

		//Limit this as we may end up with bFailedTranslocation again and don't want to spam
		NextTranslocHackTime = WorldInfo.TimeSeconds + 0.5;
	}

	//Always hop if possible
	bTranslocatorHop = !bFailedTranslocation;
	NextTranslocTime = (bFailedTranslocation)
		? WorldInfo.TimeSeconds + 1.0
		: FMin(NextTranslocTime, WorldInfo.TimeSeconds + 0.5);
}

//fox: Dodge! Jump! Run over stuff! (really for Manta and Scavenger, with some Fury hacks thrown in)
function TimedJumpDodgeHack()
{
	local int i;
	local float Dist;

	//This timer is modified by ExecuteWhatToDoNext, so reset it
	SetTimer(10.0 / FMax(0.1, Skill * FRand() + Jumpiness), false, 'TimedJumpDodgeHack');

	if (Pawn == None || MoveTarget == None)
		return;

	Dist = VSize(MoveTarget.Location * vect(1,1,0) - Pawn.Location * vect(1,1,0));
	if (Pawn.bStationary || Stopped()
	|| Pawn.ReachedDestination(MoveTarget)
	|| (Pawn(MoveTarget) == None && Dist < FMin(Pawn.GroundSpeed, Pawn.AirSpeed))
	|| (Pawn(MoveTarget) != None && Dist < CurrentFormationSize + 128.0
		&& WorldInfo.GRI.OnSameTeam(Pawn, MoveTarget))
	|| LiftCenter(MoveTarget) != None || LiftExit(MoveTarget) != None
	|| LiftCenter(Pawn.Anchor) != None || LiftExit(Pawn.Anchor) != None
	|| (PendingMover != None && !IsZero(PendingMover.Velocity))
	|| FRand() > 0.1 * Skill + 0.3 * Jumpiness)
		return;

	if (UTPawn(Pawn) != None && Dist >= UTPawn(Pawn).DodgeSpeed) {
		if (Pawn.Physics == PHYS_Walking
		&& Pawn.Weapon != None && !Pawn.Weapon.bMeleeWeapon
		&& FRand() < 0.3 + 0.2 * Jumpiness) {
			bNotifyApex = true;
			bPendingDoubleJump = (UTPawn(Pawn).JumpBootCharge < 1 && FRand() < 0.5 + 0.2 * Jumpiness);
			Pawn.DoJump(false);
		}
		MayDodgeToMoveTarget();
	}
	else if (UTVehicle(Pawn) != None) {
		i = Rand(3);
		if (UTVehicle_Fury(Pawn) != None) {
			//It'd be nice if enums supported mapping by int
			switch (i) {
				case 1: UTVehicle_Fury(Pawn).BoostAttempt = EBD_Right; break;
				case 2: UTVehicle_Fury(Pawn).BoostAttempt = EBD_Left; break;
				default: UTVehicle_Fury(Pawn).BoostAttempt = EBD_Forward;
			}
			UTVehicle_Fury(Pawn).ServerBoost(UTVehicle_Fury(Pawn).BoostAttempt);
		}
		switch(i) {
			case 2: UTVehicle(Pawn).SetInputs(0.0, 0.0, -1.0); break;
			case 1: UTVehicle(Pawn).SetInputs(0.0, 0.0, 1.0); break;
			default: UTVehicle(Pawn).SetInputs(0.0, 0.0, 0.0);
		}
	}
}

//fox: Additional MayDodgeToMoveTarget that runs more often for PHYS_Falling
function TimedWallDodgeHack()
{
	if (Pawn != None && Pawn.Physics == PHYS_Falling)
		MayDodgeToMoveTarget();
}

//fox: Additional hoverboard usage check outside of main loop if on hoverboard
function TimedCheckHoverboard()
{
	if (Squad != None && UTVehicle_Hoverboard(Pawn) != None)
		Squad.CheckHoverboard(Self);
}

//fox: For Scavenger so we can safely update ball mode - MUST be called on a timer so we don't mess with physics
function TimedUpdateScavengerBallMode()
{
	local UTVehicle_Scavenger V;

	V = UTVehicle_Scavenger(Pawn);
	if (V != None) {
		V.BallStatus.bIsInBallMode = !V.BallStatus.bIsInBallMode;
		V.BallStatus.bBoostOnTransition = true;
		V.BallModeTransition();
	}
}

//fox: Clean up avoidance mapper if appropriate
function Destroyed()
{
	local UTBot B;

	//Unregister our various timers
	UnRegisterTimers();

	//Check if there are no bots in play -- HACK don't use NumBots because we don't increment it for FakeSpectator bots
	foreach WorldInfo.AllControllers(class'UTBot', B)
		if (B != Self)
			break;
	if ((B == None || B == Self) && AvoidanceMapper != None && !AvoidanceMapper.bDeleteMe)
		AvoidanceMapper.Destroy();

	Super.Destroyed();
}

//fox: Say what you're doing when you acknowledge
function SetBotOrders(name NewOrders, Controller OrderGiver, bool bShouldAck)
{
	local string AckMsg;

	Super.SetBotOrders(NewOrders, OrderGiver, bShouldAck);

	//Actually, don't modify aggressiveness
	Aggressiveness = BaseAggressiveness;

	if (bShouldAck) {
		if (NewOrders == 'Follow') {
			SendMessage(OrderGiver.PlayerReplicationInfo, 'GOTYOURBACK', 5);
			bInitLifeMessage = true;
		}
		else {
			if (UTPlayerController(OrderGiver) != None)
				UTPlayerController(OrderGiver).LastCombatUpdateTime = -1000;
			SendMessage(OrderGiver.PlayerReplicationInfo, 'STATUS', 5);
		}

		if (PlayerController(OrderGiver) != None) {
			//Format NewOrders output to something like this: ATTACK -> Attacking, Freelance -> Freelancing
			AckMsg = GetHumanReadableName() $ ":" @ Caps(Left(NewOrders, 1)) $ Locs(Mid(NewOrders, 1, Len(NewOrders) - ((Right(NewOrders, 1) ~= "e") ? 2 : 1))) $ "ing";
			AckMsg $= (Squad.SquadLeader == Self)
				? ", leading squad"
				: ", joining squad lead by" @ Squad.SquadLeader.GetHumanReadableName();
			PlayerController(OrderGiver).ClientMessage(AckMsg);
		}
	}
}

//fox: Check if there's someone we should toss a flag to
function CheckFlagTransfer()
{
	local float Rating, BestRating;
	local Controller C, BestC;

	if (Pawn == None || !PlayerReplicationInfo.bHasFlag || WorldInfo.TimeSeconds - ForcedFlagDropTime < FlagTossInterval)
		return;

	//Find the best possible toss
	BestRating = 0.5;
	foreach WorldInfo.AllControllers(class'Controller', C) {
		if (C != Self && C.Pawn != None && C.Pawn.bCanPickupInventory //Infers Vehicle(C.Pawn) == None && !UTPawn(C.Pawn).IsHero()
		&& VSize(C.Pawn.Location - Pawn.Location) < MaxTossDist
		&& FastTrace(C.Pawn.Location, Pawn.Location)
		&& WorldInfo.GRI.OnSameTeam(Self, C)
		&& (RouteGoal == None || VSize(C.Pawn.Location - RouteGoal.Location) < VSize(Pawn.Location - RouteGoal.Location))) {
			Rating = RelativeStrength(C.Pawn);
			if (Rating > BestRating) {
				BestRating = Rating;
				BestC = C;
			}
		}
	}

	if (BestC != None)
		TossFlagTo(BestC);
}

//fox: Check if there's someone we should toss a weapon to
function CheckWeaponTransfer()
{
	local float Rating, BestRating;
	local Controller C, BestC;
	local Weapon Weap;

	if (Pawn == None || WorldInfo.TimeSeconds - LastWeaponTossTime < WeaponTossInterval || (Enemy != None && LineOfSightTo(Enemy)))
		return;

	//Find the best possible toss - also toss enforcers to those using one for akimbo
	BestRating = -0.95;
	foreach WorldInfo.AllControllers(class'Controller', C) {
		if (C != Self && C.Pawn != None && C.Pawn.bCanPickupInventory //Infers Vehicle(C.Pawn) == None && !UTPawn(C.Pawn).IsHero()
		&& VSize(C.Pawn.Location - Pawn.Location) < MaxTossDist
		&& FastTrace(C.Pawn.Location, Pawn.Location)
		&& WorldInfo.GRI.OnSameTeam(Self, C)) {
			//Check enforcer tosses
			if (BestC == None && UTWeap_Enforcer(C.Pawn.Weapon) != None && UTWeap_Enforcer(Pawn.Weapon) == None) {
				BestC = C;
				continue;
			}

			//See if this pawn already has our weapon and has ammo for it, and skip this pawn if so
			Weap = Weapon(C.Pawn.FindInventoryType(Pawn.Weapon.Class));
			if (Weap != None && Weap.HasAnyAmmo())
				continue;

			Rating = RelativeStrength(C.Pawn);
			if (Rating < BestRating) {
				BestRating = Rating;
				BestC = C;
			}
		}
	}

	if (BestC != None)
		TossWeaponTo(BestC);
}

//fox: Toss flag to target (does NOT call super function)
function TossFlagTo(Controller OrderGiver)
{
	local UTPawn BotPawn;
	local UTPlayerReplicationInfo PRI;
	local UTCarriedObject Flag;

	PRI = UTPlayerReplicationInfo(Pawn.PlayerReplicationInfo);
	if (PRI == None)
		return;

	BotPawn = UTPawn(Pawn);
	if (BotPawn == None && UTVehicle(Pawn) != None)
		BotPawn = UTPawn(UTVehicle(Pawn).Driver);
	if (BotPawn == None)
		return;

	Flag = PRI.GetFlag();
	if (Flag == None)
		return;

	//Toss the flag towards the target
	if (VSize(OrderGiver.Pawn.Location - Pawn.Location) < MaxTossDist) {
		GoalString = "Tossing Flag to" @ OrderGiver;
		StopMovement();
		MoveTarget = None;
		Flag.Drop();
		Flag.Velocity = vect(0,0,200) + (OrderGiver.Pawn.Location - Pawn.Location) * TossVelocityMult + OrderGiver.Pawn.Velocity;
		ForcedFlagDropTime = WorldInfo.TimeSeconds;
	}
}

//fox: Toss weapon to target
function TossWeaponTo(Controller OrderGiver)
{
	local UTPawn BotPawn;
	local Weapon TossedGun;

	BotPawn = UTPawn(Pawn);
	if (BotPawn == None)
		return;

	//if no weapon or we can't throw it, throw out enforcer instead - also toss enforcers to those using one for akimbo
	TossedGun = BotPawn.Weapon;
	if (TossedGun == None || !TossedGun.CanThrow()
	|| (UTWeap_Enforcer(OrderGiver.Pawn.Weapon) != None && UTWeap_Enforcer(Pawn.Weapon) == None)) {
		TossedGun = Weapon(BotPawn.FindInventoryType(class'UTWeap_Enforcer'));
		if (TossedGun == None || !TossedGun.CanThrow())
			return;
	}

	//Toss the weapon towards the target
	if (VSize(OrderGiver.Pawn.Location - Pawn.Location) < MaxTossDist) {
		GoalString = "Tossing Weapon to" @ OrderGiver;
		BotPawn.TossWeapon(TossedGun, vect(0,0,200) + (OrderGiver.Pawn.Location - Pawn.Location) * TossVelocityMult + OrderGiver.Pawn.Velocity);
		LastWeaponTossTime = WorldInfo.TimeSeconds;
		SwitchToBestWeapon();
	}
}

//fox: From UTPlayerController, sex is set when mesh is set for pawn
function UpdateSex() {}

//fox: Fix missing hit effect for ViewTargets that aren't our Pawn (such as when spectating)
function NotifyTakeHit(Controller InstigatedBy, vector HitLocation, int Damage, class<DamageType> damageType, vector Momentum)
{
	local PlayerController PC;

	Super.NotifyTakeHit(InstigatedBy, HitLocation, Damage, damageType, Momentum);

	//Also broadcast to anyone watching us - this should match foxPlayerController
	foreach WorldInfo.AllControllers(class'PlayerController', PC)
		if (PC != Self && PC.GetViewTarget() == Pawn)
			PC.NotifyTakeHit(InstigatedBy, HitLocation, Damage, damageType, Momentum);
}

//fox: Determine if we're near a teammate e.g. should send 'INCOMING' instead of calling for backup
function bool IsNearbyTeammate()
{
	local Controller C;

	foreach WorldInfo.AllControllers(class'Controller', C)
		if (C != Self && C.Pawn != None
		&& (C.Pawn == Pawn
			|| (VSize(C.Pawn.Location - Pawn.Location) < class'foxSquadAI'.default.FormationSize * 1.2
				&& FastTrace(C.Pawn.Location, Pawn.Location)))
		&& WorldInfo.GRI.OnSameTeam(Self, C))
			return true;
	return false;
}

//fox: Play a taunt emote without voice (as this is handled separately now) - this should match foxPlayerController
function Celebrate()
{
	ServerCelebrate();
}
reliable server function ServerCelebrate()
{
	if (UTPawn(Pawn) != None)
		UTPawn(Pawn).PlayEmote(UTPawn(Pawn).TauntNames[Rand(6)], -1);
}

//fox: Delay auto-taunts a little (a little variance and gives PRI time to do a global taunt instead) - this should match foxPlayerController
function SendMessage(PlayerReplicationInfo Recipient, name MessageType, float Wait, optional class<DamageType> DamageType)
{
	//Bot Only: Change 'STATUS' etc. to 'INCOMING' if we're near a teammate - TODO do something similar for players
	if (Enemy != None && (MessageType == 'STATUS' || MessageType == 'INJURED') && IsNearbyTeammate())
		MessageType = 'INCOMING';

	// don't autotaunt people who don't want it
	if (MessageType == 'TAUNT' && Recipient != None && UTPlayerController(Recipient.Owner) != None && !UTPlayerController(Recipient.Owner).bAutoTaunt)
		return;

	// limit frequency of same message, unless it's an auto-taunt (manual taunts don't use SendMessage)
	if (MessageType != 'TAUNT' && MessageType == OldMessageType && WorldInfo.TimeSeconds - OldMessageTime < Wait)
		return;

	//Delay auto-taunts a little (a little variance and gives PRI time to do a global taunt instead)
	if (MessageType == 'TAUNT') {
		TRecipient = Recipient;
		TMessageType = MessageType;
		TDamageType = DamageType;
		SetTimer(0.2 + FRand(), false, 'TimedTaunt');
		return;
	}

	UTPlayerReplicationInfo(PlayerReplicationInfo).VoiceClass.static.SendVoiceMessage(Self, Recipient, MessageType, DamageType);
	OldMessageType = MessageType;
	OldMessageTime = WorldInfo.TimeSeconds;
}
function TimedTaunt()
{
	local UTPlayerReplicationInfo UTPRI;
	local UTPlayerController PC;
	local int MsgIdx;

	//If it's round-ending / waiting state, force global and celebrate - we are auto-taunting, after all ;)
	if (WorldInfo.Game.bGameEnded || WorldInfo.Game.bWaitingToStartMatch) {
		TRecipient = None;
		Celebrate();
	}

	UTPRI = UTPlayerReplicationInfo(PlayerReplicationInfo);

	//HACK Force taunt to all players instead of only teammates, which is what UTVoice does if we're on a team
	//This won't be an issue if we're using a foxVoice VoiceClass, but we might not be, so it's here anyway
	if (TRecipient == None) {
		MsgIdx = UTPRI.VoiceClass.static.GetMessageIndex(Self, None, TMessageType, TDamageType);
		foreach WorldInfo.AllControllers(class'UTPlayerController', PC)
			PC.ReceiveBotVoiceMessage(UTPRI, MsgIdx, None);
	}
	else
		UTPRI.VoiceClass.static.SendVoiceMessage(Self, TRecipient, TMessageType, TDamageType);
	OldMessageType = TMessageType;
	OldMessageTime = WorldInfo.TimeSeconds;
}

//fox: Easter egg!
function ResetSkill()
{
	//Use badass bots in campaign mode
	if (UTGameReplicationInfo(WorldInfo.Game.GameReplicationInfo).bStoryMode
	&& Squad.Team != None && Squad.Team.TeamIndex == 0) {
		switch (GetHumanReadableName()) {
			case "Reaper":
			case "Bishop":
			case "Othello":
			case "Jester":
				bAlphaAndOmega = true;
		}
	}

	//Also use badass bots for FakeSpectate, because players are badass or something
	bAlphaAndOmega = bAlphaAndOmega || !PlayerReplicationInfo.bBot;

	//Crank those stats! Reckless aggression! Ballin' aiming! Hot feet!
	if (bAlphaAndOmega) {
		//`Log("foxLog: Setting badass bot" @ Self @ GetHumanReadableName());
		Aggressiveness = 1.0;
		BaseAggressiveness = Aggressiveness;
		Accuracy = 5.0;
		StrafingAbility = 5.0;
		CombatStyle = 1.0;
		Jumpiness = 1.0;
		Tactics = 5.0;
		ReactionTime = 5.0;
	}

	Super.ResetSkill();
}

//fox: Always DoTacticalMove if in vehicle
function DoTacticalMove()
{
	if (Vehicle(Pawn) != None) {
		GotoState('TacticalMove');
		return;
	}
	Super.DoTacticalMove();
}

//fox: Simpler VehicleFightEnemy for improved movement and fewer gunner glitches
function VehicleFightEnemy(bool bCanCharge, float EnemyStrength)
{
	local UTVehicle V;

	V = UTVehicle(Pawn);
	if (V == None)
		V = UTVehicle(Pawn.GetVehicleBase());

	if (V != None && V.bShouldLeaveForCombat) {
		LeaveVehicle(true);
		return;
	}

	if (!bFrustrated && Pawn.HasRangedAttack() && Pawn.TooCloseToAttack(Enemy)) {
		GoalString = "Retreat";
		DoRetreat();
		return;
	}

	if (!LineOfSightTo(Enemy)) {
		if (Squad.MustKeepEnemy(Enemy)) {
			GoalString = "Hunt priority enemy";
			GotoState('Hunting');
			return;
		}
		//TODO HACK Just keep driving towards objective -- this should really be cleaned up later!
		if (RouteGoal != None && !Pawn.ReachedDestination(RouteGoal) && SetRouteToGoal(RouteGoal)) {
			GoalString = "Keep driving to goal while hunting";
			return;
		}
		if (!bCanCharge || (Squad.IsDefending(Self) && LostContact(4))) {
			GoalString = "Stake Out";
			DoStakeOut();
		}
		else if (CanStakeOut() && (IsSniping() || (Aggression < 1 && !LostContact(3+2*FRand())))) {
			GoalString = "Stake Out2";
			DoStakeOut();
		}
		else {
			GoalString = "Hunt";
			GotoState('Hunting');
		}
		return;
	}

	BlockedPath = None;
	Focus = Enemy;

	if (V != None && V.RecommendCharge(Self, Enemy)) {
		GoalString = "Charge";
		DoCharge();
		return;
	}

	//Get out of Scavenger ball mode
	if (UTVehicle_Scavenger(Pawn) != None
	&& UTVehicle_Scavenger(Pawn).BallStatus.bIsInBallMode)
		SetTimer(0.01, false, 'TimedUpdateScavengerBallMode');

	//TODO HACK Just keep driving towards objective -- this should really be cleaned up later!
	if (RouteGoal != None && !Pawn.ReachedDestination(RouteGoal) && SetRouteToGoal(RouteGoal)) {
		GoalString = "Keep driving to goal while attacking";
		return;
	}

	//If stationary turret / gunning start running timed WhatToDoNexts
	if (V != None && (V.bStationary || V.bIsOnTrack || V.Controller != Self)) {
		GoalString = "Do turret stuff";
		SetTimer(1.0, false, 'WhatToDoNext');
		return;
	}

	GoalString = "Do tactical move";
	DoTacticalMove();
}

//fox: Always use TimedFireWeaponAtEnemy (set by SetCombatTimer)
function Timer() { TimedFireWeaponAtEnemy(); }
state MoveToGoal
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state MoveToGoalWithEnemy
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state Charging
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state VehicleCharging
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state Hunting
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state RangedAttack
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}
state WaitingForLanding
{
	function Timer() { TimedFireWeaponAtEnemy(); }
}

//fox: Catch-all to make MinUndeployTime work (workaround to fix issues with Leviathan etc.), called after ExecuteWhatToDoNext / CanAttack
function DeployHack()
{
	//CachedDeployableVehicle != Pawn won't cover all cases, but it's still a reasonable optimization over calling GetDeployableVehicle every time
	if (CachedDeployableVehicle != Pawn)
		CachedDeployableVehicle = GetDeployableVehicle();
	if (CachedDeployableVehicle != None && !CanDeployVehicle(CachedDeployableVehicle))
		CachedDeployableVehicle.ClearTimer('ServerToggleDeploy');
}

//fox: Don't get stuck in deploy/undeploy
function bool CanDeployVehicle(UTVehicle_Deployable V)
{
	return MinUnDeployTime == -1 || V == None || V.IsDeployed()
		|| WorldInfo.TimeSeconds - FMax(VehicleEnterTime, V.LastDeployStartTime) > V.UnDeployTime + MinUnDeployTime;
}
function bool WaitToDeploy()
{
	if (!CanDeployVehicle(GetDeployableVehicle()))
		return false;
	return Super.WaitToDeploy();
}
state Defending
{
	function TryToDeploy()
	{
		if (!CanDeployVehicle(GetDeployableVehicle()))
			return;
		Super(Defending).TryToDeploy();
	}
}

//fox: Don't leave a vehicle until MinVehicleTime has passed (for blocking version)
state LeavingVehicle
{
Begin:
	if (CanLeaveVehicle()) {
		TryLeavingVehicle();
		if (Vehicle(Pawn) != None) {
			// failed to leave vehicle
			// possibly no room, so try moving around
			GoalString = "Failed to exit vehicle - try moving to more open area";
			FindRandomDest();
			if (RouteCache.length > 0)
				MoveToward(RouteCache[0]);
		}
	}
	else
		SetRouteToGoal((RouteGoal != None) ? RouteGoal : FindRandomDest());

	LatentWhatToDoNext();
	if (bSoaking)
		SoakStop("STUCK IN LEAVINGVEHICLE");
}

//fox: Always call global NotifyHitWall in specific states
state TacticalMove
{
	function Timer() { TimedFireWeaponAtEnemy(); }

	function bool NotifyHitWall(vector HitNormal, actor Wall)
	{
		Global.NotifyHitWall(HitNormal, Wall);
		return Super(TacticalMove).NotifyHitWall(HitNormal, Wall);
	}
}
state FindAir
{
	function Timer() { TimedFireWeaponAtEnemy(); }

	function bool NotifyHitWall(vector HitNormal, actor Wall)
	{
		Global.NotifyHitWall(HitNormal, Wall);
		return Super(FindAir).NotifyHitWall(HitNormal, Wall);
	}
}

//fox: Don't wait as long in StakeOut
state StakeOut
{
	function Timer() { TimedFireWeaponAtEnemy(); }

Begin:
	Focus = None;
	FocusOnLeader(false);
	if (WaitToDeploy())
		Sleep(UTVehicle_Deployable(Pawn).DeployTime + 0.2);
	CheckIfShouldCrouch(Pawn.Location,FocalPoint, 1);
	FinishRotation();
	if (FocusOnLeader(false))
		FireWeaponAt(Focus);
	else if (Vehicle(Pawn) != None
	|| (Pawn.Weapon != None && !Pawn.Weapon.bMeleeWeapon
	&& Squad.ShouldSuppressEnemy(Self) && ClearShot(FocalPoint,true)))
		WeaponFireAgain(false);
	else
		StopFiring();
	Sleep(FRand());

	if (Pawn.bIsCrouched
	&& !FastTrace(FocalPoint, Pawn.Location + Pawn.EyeHeight * vect(0,0,1))
	&& FastTrace(FocalPoint, Pawn.Location + Pawn.GetCollisionHeight() * vect(0,0,1)))
		Pawn.bWantsToCrouch = false;
	LatentWhatToDoNext();
	if (bSoaking)
		SoakStop("STUCK IN STAKEOUT!");
}

//fox: Don't check if we're stuck when dead, and make FearCost added to points permanent for more dynamic pathfinding
state Dead
{
	function bool CheckIfStuck() { return false; }
	function TimedUpdateEnemy() {}
	function TimedUpdateFocus() {}
	function TimedJumpDodgeHack() {}
	function TimedCheckHoverboard() {}
	function TimedUpdateScavengerBallMode() {}

	function BeginState(Name PreviousStateName)
	{
		local NavigationPoint NP;

		CachedDefensePoint = DefensePoint;
		NP = NavigationPoint(MoveTarget);

		Super(Dead).BeginState(PreviousStateName);

		if (NP != None)
			NP.FearCost = FMin(NP.FearCost * DeathFearCostScaleFactor, class'NavigationPoint'.const.INFINITE_PATH_COST);
		WorldInfo.Game.bDoFearCostFallOff = !bDeathFearCostAccumulate;
	}
}

//fox: Don't check if we're stuck in round-end, and celebrate with voice!
state RoundEnded
{
	function bool CheckIfStuck() { return false; }
	function TimedUpdateEnemy() {}
	function TimedUpdateFocus() {}
	function TimedJumpDodgeHack() {}
	function TimedCheckHoverboard() {}
	function TimedUpdateScavengerBallMode() {}

	function BeginState(Name PreviousStateName)
	{
		Super(RoundEnded).BeginState(PreviousStateName);

		//Unregister our various timers
		UnRegisterTimers();

		//Start a randomly delayed CelebrateVictory timer instead
		SetTimer(((Pawn == UTGame(WorldInfo.Game).EndGameFocus) ? 1.0 : 3.0) + FRand() * 4.0, false, 'CelebrateVictory');

		//Also toss our AvoidanceMapper to save that little bit of performance in round-ending state :)
		if (AvoidanceMapper != None)
			AvoidanceMapper.Destroy();
	}

	function Celebrate()
	{
		Global.Celebrate();
	}

	function CelebrateVictory()
	{
		local TeamInfo TI;
		local PlayerReplicationInfo PRI;

		//Don't celebrate if it's not an ended game :) (fixes some amusing bugs such as continuing to taunt in next match)
		if (!WorldInfo.Game.bGameEnded)
			return;

		//Don't celebrate if we don't have endgame focus and don't have highest score
		if (Pawn == None || Pawn != UTGame(WorldInfo.Game).EndGameFocus) {
			if (PlayerReplicationInfo.Team != None) {
				foreach WorldInfo.GRI.Teams(TI)
					if (PlayerReplicationInfo.Team.Score < TI.Score)
						return;
			}
			else
				foreach WorldInfo.GRI.PRIArray(PRI)
					if (PlayerReplicationInfo.Score < PRI.Score)
						return;
		}

		//Taunt! And maybe taunt again
		SendMessage(None, 'TAUNT', 3);
		if (FRand() < 0.33)
			SetTimer(0.5 + FRand() * 4.0, false, 'CelebrateVictory');
	}
}

//fox: Don't check if we're stuck if we're kinda supposed to be
state Frozen
{
	function bool CheckIfStuck() { return false; }
	function TimedUpdateEnemy() {}
	function TimedUpdateFocus() {}
	function TimedJumpDodgeHack() {}
	function TimedCheckHoverboard() {}
	function TimedUpdateScavengerBallMode() {}
}
state FrozenMovement
{
	function Timer() { TimedFireWeaponAtEnemy(); }

	function bool CheckIfStuck() { return false; }
	function TimedJumpDodgeHack() {}
	function TimedCheckHoverboard() {}
	function TimedUpdateScavengerBallMode() {}
}

defaultproperties
{
	bAllowRouteReuse=True //Must be true so vehicles don't get slowed down with changing paths
	InUseNodeCostMultiplier=1.0
	PreferredDoorWeapons.Add(class'UTWeap_Enforcer')
	PreferredDoorWeapons.Add(class'UTWeap_Stinger')
	PreferredDoorWeapons.Add(class'UTWeap_LinkGun')
	PreferredDoorWeapons.Add(class'UTWeap_ShockRifle')
	PreferredDoorWeapons.Add(class'UTWeap_SniperRifle')
	PreferredDoorWeapons.Add(class'UTWeap_RocketLauncher')
	PreferredDoorWeapons.Add(class'UTWeap_Avril')
	bForceAlternateTargetLocations=True
	ExtendedVehiclePathingLevel=1
	AvoidanceMappingLevel=2
	bAvoidanceMappingUsePlayerCount=False
	bAvoidanceMappingAvoidTeammates=False
	bAvoidanceMappingUpdateVehicles=False
	AvoidanceMappingBuildInterval=2.0
	AvoidanceMappingSearchRadius=16384.0
	bAvoidanceMappingFullTrace=True
	DeathFearCostScaleFactor=2
	bDeathFearCostAccumulate=False
	MinVehicleTime=6.0
	MinUnDeployTime=4.0
	CheckIfStuckInterval=2.0
	CheckIfStuckRadius=128.0
	UnstuckTargetInterval=3.0
	UnstuckTargetMinRadius=128.0
	UnstuckNudgeVelocity=400.0
	bSwimmingHitWallFix=true
	bAvoidSwimming=true
	MaxSightRadius=16384.0
	MinForceNeedWeaponWeight=0.0
	MinForceWantInventoryWeight=0.0006
	WeaponTossInterval=60.0
	FlagTossInterval=10.0
	MaxTossDist=512.0
	TossVelocityMult=2.0
	RandomOrdersMaxDefenders=2
	RandomOrdersMaxAttackers=-1
	RandomOrdersMaxFollowers=0
	RandomOrdersMinReassignTime=120.0
	RandomOrdersMaxReassignTime=900.0
	bLoneFreelanceSquads=False
	MaxSquadSize=32
	MaxSquadRoutes=5
	bAlphaAndOmega=False
}
