/**
 * Simple SquadAI override ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxCTFSquadAI extends UTCTFSquadAI;

`define CTFClass
`include(foxSquadAICommonVars.uci);

//fox: Modified to get rid of the IsHumanControlled check (only present in CTFSquadAI, interestingly)
function float ModifyThreat(float current, Pawn NewThreat, bool bThreatVisible, UTBot B)
{
	if (NewThreat.PlayerReplicationInfo != None
	&& NewThreat.PlayerReplicationInfo.bHasFlag
	&& bThreatVisible) {
		if (VSize(B.Pawn.Location - NewThreat.Location) < 1500
		|| (B.Pawn.Weapon != None && UTWeapon(B.Pawn.Weapon).bSniping)
		|| VSize(NewThreat.Location - EnemyFlag.HomeBase.Location) < 2000)
			return current + 6;
		else
			return current + 1.5;
	}
	//else if ( NewThreat.IsHumanControlled() )
	//	return current + 0.1;
	else
		return current;
}

//fox: Treat friendly holding flag as priority objective
function byte PriorityObjective(UTBot B)
{
	local byte ret;

	ret = Super.PriorityObjective(B);
	if (EnemyFlag != None && EnemyFlag.Holder != None)
		return 1;
	return ret;
}

//fox: FlagCarrier Accessed None fix, always chase flag when defending, use TellBotToFollow for escort
function bool CheckSquadObjectives(UTBot B)
{
	local bool bSeeFlag;
	local Controller FlagCarrier;

	if (B.PlayerReplicationInfo.bHasFlag)
		return OrdersForFlagCarrier(B);

	// Heroes can't pick up flags
	if (!B.Pawn.bCanPickupInventory && UTPawn(B.Pawn) != None && UTPawn(B.Pawn).IsHero()) {
		if (FriendlyFlag.Holder != None) {
			if (B.bPursuingFlag)
				return TryToIntercept(B,FriendlyFlag.Holder,EnemyFlag.Homebase);
			bSeeFlag = B.LineOfSightTo(FriendlyFlag.Position());
			if (bSeeFlag) {
				if (B.Enemy == None || (B.Enemy.PlayerReplicationInfo != None && !B.Enemy.PlayerReplicationInfo.bHasFlag))
					FindNewEnemyFor(B,(B.Enemy != None) && B.LineOfSightTo(B.Enemy));
				if (WorldInfo.TimeSeconds - LastSeeFlagCarrier > 6) {
					LastSeeFlagCarrier = WorldInfo.TimeSeconds;
					B.SendMessage(None, 'ENEMYFLAGCARRIERHERE', 14);
				}
				B.GoalString = "Attack enemy flag carrier";
				B.bPursuingFlag = true;
				return TryToIntercept(B,FriendlyFlag.Holder,EnemyFlag.Homebase);
			}
		}
		if (VSize(B.Pawn.Location - EnemyFlag.Position().Location) > 1000)
			return FindPathToObjective(B,EnemyFlag.Position());
		return false;
	}
	//AddTransientCosts(B,1); //This is no longer needed due to AddDynamicCosts ~fox

	if (EnemyFlag.Holder != None) {
		FlagCarrier = EnemyFlag.Holder.Controller;
		if (FlagCarrier == None && EnemyFlag.Holder.DrivenVehicle != None)
			FlagCarrier = EnemyFlag.Holder.DrivenVehicle.Controller;

		// check if should tow flag carrier back to base
		if (FlagCarrier != None && UTVehicle(FlagCarrier.Pawn) != None
		&& UTVehicle(FlagCarrier.Pawn).GetTowingVehicle() == B.Pawn //Added FlagCarrier != None check ~fox
		&& CheckTowing(B, UTVehicle(B.Pawn)))
			return true;
	}

	if (!FriendlyFlag.bHome) {
		bSeeFlag = B.LineOfSightTo(FriendlyFlag.Position());
		if (Team.Size == 1) {
			if (CheckVehicle(B))
				return true;
			if (B.NeedWeapon() && B.FindInventoryGoal(0)) {
				B.SetAttractionState();
				return true;
			}

			// keep attacking if 1-0n-1
			if (FriendlyFlag.Holder != None || VSize(B.Pawn.Location - FriendlyFlag.Position().Location) > VSize(B.Pawn.Location - EnemyFlag.Position().Location))
				return FindPathToObjective(B,EnemyFlag.Position());
		}
		if (bSeeFlag) {
			if (FriendlyFlag.Holder == None)
				if (GoPickupFlag(B))
					return true;
			else {
				if (B.Enemy == None || (B.Enemy.PlayerReplicationInfo != None && !B.Enemy.PlayerReplicationInfo.bHasFlag))
					FindNewEnemyFor(B,(B.Enemy != None) && B.LineOfSightTo(B.Enemy));
				if (WorldInfo.TimeSeconds - LastSeeFlagCarrier > 6) {
					LastSeeFlagCarrier = WorldInfo.TimeSeconds;
					B.SendMessage(None, 'ENEMYFLAGCARRIERHERE', 14);
				}
				B.GoalString = "Attack enemy flag carrier";
				if (B.IsSniping())
					return false;
				B.bPursuingFlag = true;
				return TryToIntercept(B,FriendlyFlag.Holder,EnemyFlag.Homebase);
			}
		}

		if (GetOrders() == 'Attack') {
			// break off attack only if needed
			if (B.bPursuingFlag || bSeeFlag || B.LastRespawnTime > FriendlyFlag.TakenTime || NearHomeBase(B)
			|| (WorldInfo.TimeSeconds - FriendlyFlag.TakenTime > UTCTFBase(FriendlyFlag.HomeBase).BaseExitTime && !NearEnemyBase(B))) {
				//Attackers should only chase enemy flag carrier if your team has the flag ~fox
				if (EnemyFlag.Holder != None) {
					if (B.Enemy == None && CheckVehicle(B))
						return true;
					B.bPursuingFlag = true;
					B.GoalString = "Go after enemy holding flag rather than attacking";
					if (FriendlyFlag.Holder != None)
						return TryToIntercept(B,FriendlyFlag.Holder,EnemyFlag.Homebase);
					else if (GoPickupFlag(B))
						return true;
				}
			}
			else if (B.bReachedGatherPoint)
				B.GatherTime = WorldInfo.TimeSeconds - 10;
		}
		//chase flag even if defending! Unless someone's carrying the enemy flag ~fox
		else if (PlayerController(SquadLeader) == None
		//&& (CurrentOrders != 'Defend' || bSeeFlag || B.bPursuingFlag || FlagNearBase())
		//&& !B.IsSniping()) {
		&& (GetOrders() != 'Defend' || EnemyFlag.Holder == None)) {
			if (B.Enemy == None && CheckVehicle(B))
				return true;
			// FIXME - try to leave one defender at base
			B.bPursuingFlag = true;
			B.GoalString = "Go find my flag";
			if (FriendlyFlag.Holder != None)
				return TryToIntercept(B,FriendlyFlag.Holder,EnemyFlag.Homebase);
			else if (GoPickupFlag(B))
				return true;
		}
	}
	B.bPursuingFlag = false;

	if (SquadObjective == EnemyFlag.Homebase && B.Enemy != None && FriendlyFlag.Homebase.BotNearObjective(B)
	&& (WorldInfo.TimeSeconds - B.LastSeenTime < 3)) {
		if (!EnemyFlag.bHome && EnemyFlag.Holder == None && B.LineOfSightTo(EnemyFlag.Position()))
			return FindPathToObjective(B,EnemyFlag.Position());
		B.GoalString = "Intercept incoming enemy!";
		return false;
	}

	if (EnemyFlag.Holder == None && (EnemyFlag.bHome || WorldInfo.TimeSeconds - B.ForcedFlagDropTime > 8)) {
		if (!EnemyFlag.bHome || EnemyFlag.Homebase.BotNearObjective(B)) {
			if (CheckVehicle(B))
				return true;
			B.GoalString = "Near enemy flag!";
			return FindPathToObjective(B,EnemyFlag.Position());
		}
	}
	else if ((GetOrders() != 'Defend' || FriendlyFlag.Holder != None) && !SquadLeader.IsA('PlayerController')) {
		if (FlagCarrier != None) { //Added FlagCarrier != None check ~fox
			// make flag carrier squad leader if on same squad
			if (SquadLeader != FlagCarrier && IsOnSquad(FlagCarrier))
				SetLeader(FlagCarrier);

			//TODO is this needed? TellBotToFollow should handle this
			//if (B.Enemy != None && B.Enemy.LineOfSightTo(FlagCarrier.Pawn)) {
			//	B.GoalString = "Fight enemy threatening flag carrier";
			//	B.FightEnemy(true,0);
			//	return true;
			//}
			//
			//if ((FlagCarrier.MoveTarget == FriendlyFlag.HomeBase
			//	|| (FlagCarrier.RouteCache.Length > 1 && FlagCarrier.RouteCache[1] == FriendlyFlag.HomeBase))
			//&& B.Enemy != None && B.LineOfSightTo(FriendlyFlag.HomeBase)) {
			//	B.GoalString = "Fight enemy while waiting for flag carrier to score";
			//	if (B.LostContact(7))
			//		B.LoseEnemy();
			//	if (B.Enemy != None) {
			//		B.FightEnemy(false,0);
			//		return true;
			//	}
			//}
		}

		if (CheckVehicle(B))
			return true;

		//Use TellBotToFollow instead of basically re-implementing it ~fox
		if (FlagCarrier != None && TellBotToFollow(B, FlagCarrier))
			return true;
	}

	return Super(UTSquadAI).CheckSquadObjectives(B);
}

`include(foxSquadAICommon.uci);
