/**
 * Simple SquadAI override ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxDMSquad extends UTDMSquad;

`include(foxSquadAICommonVars.uci);

//fox: Check an improved HuntTarget override for DM
function bool AssignSquadResponsibility(UTBot B)
{
	local Pawn HuntTarget;
	local float BestDist, NewDist;
	local Controller C;
	local Actor MoveTarget;
	local UTVehicleFactory VFactory;
	local UTVehicle V;

	if (WorldInfo.TimeSeconds - B.Pawn.CreationTime < 5.0 && B.NeedWeapon() && B.FindInventoryGoal(0.0004)) {
		B.GoalString = "Need weapon or ammo";
		B.NoVehicleGoal = B.RouteGoal;
		B.SetAttractionState();
		return true;
	}

	if (CheckVehicle(B))
		return true;

	if (B.NeedWeapon() && B.FindInventoryGoal(0)) {
		B.GoalString = "Need weapon or ammo";
		B.SetAttractionState();
		return true;
	}

	if (B.Skill > 1.25 && !InRealVehicle(B)) {
		// search for powerups
		B.RespawnPredictionTime = (B.Skill > 5.0) ? 2.0 : 0.0;

		// consider vehicles as powerups in DM
		foreach WorldInfo.AllNavigationPoints(class'UTVehicleFactory', VFactory) {
			V = VFactory.ChildVehicle;
			if ((V != None && (!V.bHasBeenDriven || V.bStationary) && VehicleDesireability(V, B) > 0.0)
			|| (V == None && VFactory.RespawnProgress < B.RespawnPredictionTime && VFactory.IsInState('Active'))) {
				B.SetRouteToGoal(VFactory);
				B.GoalString = "Get super item - vehicle" @ B.RouteGoal;
				return true;
			}
		}
		if (B.FindSuperPickup((WorldInfo.Game.NumPlayers + WorldInfo.Game.NumBots < 4) ? 6000.0 : 3500.0)) {
			B.GoalString = "Get super item" @ B.RouteGoal;
			B.SetAttractionState();
			return true;
		}
	}

	//Just in case, as we're not going to hunt players only
	B.bHuntPlayer = false;

	// if have no enemy
	if (B.Enemy == None) {
		//If we feel like hunting, then hunt both bots and players prioritizing closest sound
		if (B.Skill >= 5.0 && (B.Pawn.Weapon.AIRating > 0.7 || UTWeapon(B.Pawn.Weapon).bSniping)) {
			BestDist = B.HearingThreshold * 2.0;
			foreach WorldInfo.AllControllers(class'Controller', C) {
				//Check if our possible target has recently made noise, and if so consider them for HuntTarget
				if (C != B && C.Pawn != None && WorldInfo.TimeSeconds - FMax(C.Pawn.Noise1Time, C.Pawn.Noise2Time) < 5.0) {
					//Only accept the new HuntTarget if their sound was closer
					NewDist = VSize(B.Pawn.Location - C.Pawn.Location);
					if (NewDist < BestDist) {
						HuntTarget = C.Pawn;
						BestDist = NewDist;
					}
				}
			}
		}

		//Maybe find inventory first
		if (B.FindInventoryGoal(0)) {
			B.GoalString = "Get pickup";
			B.bHuntPlayer = false;
			B.SetAttractionState();
			return true;
		}

		//So, we actually hunting?
		if (HuntTarget != None) {
			B.GoalString = "Hunt Target" @ HuntTarget;
			//`log("foxLog: Hunting Target" @ HuntTarget);

			//If their sound was very recent, we know roughly where they are
			if (WorldInfo.TimeSeconds - FMax(HuntTarget.Noise1Time, HuntTarget.Noise2Time) < 2.0)
				B.LastKnownPosition = HuntTarget.Location;

			//If we're directly reachable, go for it - otherwise, only bother if we're heard the sound very recently
			if (B.ActorReachable(HuntTarget))
				MoveTarget = HuntTarget;
			//Fun fact, there is actually a chance the LastKnownPosition might have been set from a different possible HuntTarget - but if our current HuntTarget was in the same area, close enough
			else if (VSize(B.LastKnownPosition - HuntTarget.Location) < 500.0);
				MoveTarget = B.FindPathToward(HuntTarget,B.Pawn.bCanPickupInventory);

			if (MoveTarget != None) {
				B.MoveTarget = MoveTarget;
				if (B.CanSee(HuntTarget))
					SetEnemy(B,HuntTarget);
				B.SetAttractionState();
				return true;
			}
		}

		//Otherwise, roam around level?
		return B.FindRoamDest();
	}
	// if haven't seen enemy in a long time and don't have a ticking powerup, look for nearby inventory first
	else if (B.Skill + B.Tactics > 3.0 && B.LastSeenTime < WorldInfo.TimeSeconds - 10.0
	&& B.Pawn.FindInventoryType(class'UTTimedPowerup', true) == None && B.FindInventoryGoal(0.0005)) {
		B.GoalString = "Get pickup 2";
		B.SetAttractionState();
		return true;
	}
	return false;
}

`include(foxSquadAICommon.uci);
