/**
 * Simple SquadAI override ~fox
 * Based off and may contain code provided by and Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class foxOnslaughtSquadAI extends UTOnslaughtSquadAI;

`define ONSClass
`include(foxSquadAICommonVars.uci);
var bool bOverrideOrbObjective; //Whether to override the normal orb objective (eg. player towing orb carrier)

//fox: Do orb stuff and use CheckVehicle instead of CheckHoverboard, as CheckHoverboard does not check towing or other useful things (does NOT call the super function)
function bool CheckOrbCarrierObjective(UTBot B)
{
	if (WorldInfo.TimeSeconds - B.Pawn.CreationTime < 5.0 && B.NeedWeapon() && B.FindInventoryGoal(0.0004)) {
		B.GoalString = "Need weapon or ammo";
		B.NoVehicleGoal = B.RouteGoal;
		B.SetAttractionState();
		return true;
	}

	if (CheckVehicle(B))
		return true;

	//if bot has the orb, he wants to attack a primary objective, or defend a held enemy prime node; retain our current OrbObjective if override is still valid
	if (!bOverrideOrbObjective || OrbObjective.DefenderTeamIndex == Team.TeamIndex || !OrbObjective.PoweredBy(Team.TeamIndex)) {
		OrbObjective = ONSTeamAI.GetPriorityOrbObjectiveFor(B);

		//However we might still want to override OrbObjective with SquadObjective if appropriate (to help cap secondary nodes)
		bOverrideOrbObjective = UTOnslaughtPowernode(SquadObjective) != None && (!OrbObjective.LegitimateTargetOf(B)
			|| VSize(SquadObjective.Location - B.Pawn.Location) < VSize(OrbObjective.Location - B.Pawn.Location));
		if (bOverrideOrbObjective)
			OrbObjective = UTOnslaughtPowernode(SquadObjective);
	}

	//If the OrbObjective is hostile and it's not invulnerable to orb capture, bring it on home
	if (OrbObjective != None) {
		B.GoalString = "Take orb to" @ OrbObjective;
		if (OrbObjective.DefenderTeamIndex == Team.TeamIndex)
			return OrbObjective.TellBotHowToHeal(B);
		else if (UTOnslaughtPowernode(OrbObjective) != None
		&& UTOnslaughtPowernode(OrbObjective).Vulnerability == VS_Vulnerable)
			return FindPathToObjective(B, OrbObjective); //slam dunk!
		else
			return OrbObjective.TellBotHowToDisable(B); //for three!
	}

	return false;
}

//fox: Always take orb carrier to OrbObjective if appropriate
function Actor GetTowingDestination(UTVehicle Towed)
{
	if (Towed != None && UTBot(Towed.Controller) != None
	&& UTOnslaughtSquadAI(UTBot(Towed.Controller).Squad).OrbObjective != None)
		return UTOnslaughtSquadAI(UTBot(Towed.Controller).Squad).OrbObjective;
	return Super.GetTowingDestination(Towed);
}

//fox: Force attack on a power core if appropriate and really grab that orb
function bool CheckSquadObjectives(UTBot B)
{
	local UTOnslaughtNodeObjective Node;
	local UTGameObjective O, FirstO;
	local UTBot S;

	//Always use default DefensePriority and don't auto-adjust for core-linked powernodes, standalone nodes, etc.
	//Also here (and not just in Mutator) to catch DefensePriorities changing dynamically from UTOnslaughtNodeObjective.CheckLink() etc. -- this should match UTMutator_foxAI
	FirstO = Team.AI.Objectives;
	for (O = FirstO; O != None; O = (O.NextObjective == FirstO) ? None : O.NextObjective)
		class'UTMutator_foxAI'.static.FixDefensePriorityFor(O);

	//Always consider returning orb
	if (Team.AI.EnemyTeam.TeamFlag != None && Team.AI.EnemyTeam.TeamFlag.Holder == None
	&& VSize(Team.AI.EnemyTeam.TeamFlag.Location - B.Pawn.Location) < CloseObjectiveDist
	&& !B.Pawn.PoweredUp() && B.HasSuperWeapon() == None && !B.HasTimedPowerup()
	&& (UTPawn(B.Pawn) == None || !UTPawn(B.Pawn).IsHero())) {
		Node = UTOnslaughtPowernode(UTOnslaughtGame(WorldInfo.Game).ClosestNodeTo(Team.AI.EnemyTeam.TeamFlag));
		if (Node != None && Node.DefenderTeamIndex == Team.TeamIndex
		&& VSize(Node.Location - Team.AI.EnemyTeam.TeamFlag.Location) < CloseObjectiveDist
		&& FindPathToObjective(B, Team.AI.EnemyTeam.TeamFlag)) {
			B.GoalString = "Return enemy orb";
			return true;
		}
	}

	//Always heal friendly nodes if they need it
	FirstO = Team.AI.Objectives;
	for (O = FirstO; O != None; O = (O.NextObjective == FirstO) ? None : O.NextObjective) {
		Node = UTOnslaughtNodeObjective(O);
		if (Node != None && !Node.bIsDisabled && Node.DefenderTeamIndex == Team.TeamIndex
		&& Node.Health < ((GetOrders() == 'Defend') ? Node.DamageCapacity : Node.DamageCapacity * 0.5)
		&& VSize(Node.Location - B.Pawn.Location) < CloseObjectiveDist * 0.5)
			return Node.TellBotHowToHeal(B);
	}

	//Always pick up dropped orb if we have a node objective and it's closer
	if (UTPawn(B.Pawn) != None)
		UTPawn(B.Pawn).bJustDroppedOrb = false; //Hack to not run the similar code in super function
	if (UTOnslaughtPowernode(SquadObjective) != None
	&& B.Pawn != None && (Vehicle(B.Pawn) != None || B.Pawn.bCanPickupInventory) //Don't nab Orbs as heroes, but do as vehicles
	&& (UTVehicle(B.Pawn) == None || !UTVehicle(B.Pawn).bKeyVehicle) //... unless it's a key vehicle
	&& Team.TeamFlag != None && Team.TeamFlag.Holder == None
	&& (Team.TeamFlag.bHome || WorldInfo.TimeSeconds - B.ForcedFlagDropTime > 8)
	&& VSize(Team.TeamFlag.Location - B.Pawn.Location) < VSize(SquadObjective.Location - B.Pawn.Location) + CloseObjectiveDist) {
		//Ignore it if another squadmember is closer
		for (S = SquadMembers; S != None; S = S.NextSquadMember) {
			if (S == B || S.Pawn == None)
				continue;
			if (VSize(Team.TeamFlag.Location - S.Pawn.Location) < VSize(Team.TeamFlag.Location - B.Pawn.Location))
				break;
		}
		if (S == None) {
			if (CheckVehicle(B))
				return true;
			if (FindPathToObjective(B, Team.TeamFlag.Position())) {
				B.GoalString = "Near orb!";
				return true;
			}
		}
	}

	return Super.CheckSquadObjectives(B);
}

//fox: Handle barricades better - don't follow squadleader
function bool CheckBarricades(UTBot B)
{
	local UTGameObjective O, FirstO;
	local UTWarfareBarricade Barricade;
	local array<UTWarfareBarricade> BarricadeList;
	local int Index;
	local UTBot M;
	local bool bCoreVulnerable;

	// if already going for barricade, keep going
	Barricade = UTWarfareBarricade(SquadObjective);
	if (Barricade != None) {
		if (B.NeedWeapon() && B.FindInventoryGoal(0)) {
			B.GoalString = "Need weapon or ammo";
			B.SetAttractionState();
			return true;
		}
		else if (CheckVehicle(B) || Barricade.TellBotHowToDisable(B))
			return true;
		else if (B != SquadLeader)
			return false;
		else {
			Team.AI.FindNewObjectiveFor(Self, true);
			return false;
		}
	}

	// don't consider changing objectives if squad has flag
	for (M = SquadMembers; M != None; M = M.NextSquadMember)
		if (M.PlayerReplicationInfo.bHasFlag)
			return false;

	// see if we can go after a new barricade now
	bCoreVulnerable = ONSTeamAI.FinalCore.PoweredBy(Team.AI.EnemyTeam.TeamIndex);
	FirstO = Team.AI.Objectives;
	for (O = FirstO; O != None; O = (O.NextObjective == FirstO) ? None : O.NextObjective) {
		Barricade = UTWarfareBarricade(O);
		if (Barricade != None && Barricade.IsActive() && Barricade.DefenderTeamIndex != Team.TeamIndex
		&& (!bCoreVulnerable || Barricade.bAvalancheBarricadeHack)) {
			// if this barricade is higher priority, remove previous
			if (BarricadeList.length > 0 && Barricade.DefensePriority > BarricadeList[0].DefensePriority)
				BarricadeList.length = 0;
			BarricadeList.AddItem(Barricade);
		}
	}

	// choose one valid barricade at random
	if (BarricadeList.length > 0) {
		Index = Rand(BarricadeList.length);
		if (BarricadeList[Index].TellBotHowToDisable(B)) {
			SetObjective(BarricadeList[Index], false);
			return true;
		}
	}

	return false;
}

`include(foxSquadAICommon.uci);
